#---------------------------------------------------------------------------
# Name:        etg/ribbon_bar.py
# Author:      Robin Dunn
#
# Created:     20-Jun-2016
# Copyright:   (c) 2016-2020 by Total Control Software
# License:     wxWindows License
#---------------------------------------------------------------------------

import etgtools
import etgtools.tweaker_tools as tools

PACKAGE   = "wx"
MODULE    = "_ribbon"
NAME      = "ribbon_bar"   # Base name of the file to generate to for this script
DOCSTRING = ""

# The classes and/or the basename of the Doxygen XML files to be processed by
# this script.
ITEMS  = [ 'wxRibbonBarEvent',
           'wxRibbonPageTabInfo',
           'wxRibbonBar',
           ]

#---------------------------------------------------------------------------

def run():
    # Parse the XML file(s) building a collection of Extractor objects
    module = etgtools.ModuleDef(PACKAGE, MODULE, NAME, DOCSTRING)
    etgtools.parseDoxyXML(module, ITEMS)

    #-----------------------------------------------------------------
    # Tweak the parsed meta objects in the module object as needed for
    # customizing the generated code and docstrings.

    module.addHeaderCode('#include <wx/ribbon/bar.h>')

    c = module.find('wxRibbonBar')
    assert isinstance(c, etgtools.ClassDef)
    tools.fixWindowClass(c)
    c.find('SetArtProvider.art').transfer = True


    c = module.find('wxRibbonBarEvent')
    tools.fixEventClass(c)

    module.addPyCode("""\
        EVT_RIBBONBAR_PAGE_CHANGED    = wx.PyEventBinder(wxEVT_RIBBONBAR_PAGE_CHANGED, 1)
        EVT_RIBBONBAR_PAGE_CHANGING   = wx.PyEventBinder(wxEVT_RIBBONBAR_PAGE_CHANGING,1)
        EVT_RIBBONBAR_TAB_MIDDLE_DOWN = wx.PyEventBinder(wxEVT_RIBBONBAR_TAB_MIDDLE_DOWN, 1)
        EVT_RIBBONBAR_TAB_MIDDLE_UP   = wx.PyEventBinder(wxEVT_RIBBONBAR_TAB_MIDDLE_UP, 1)
        EVT_RIBBONBAR_TAB_RIGHT_DOWN  = wx.PyEventBinder(wxEVT_RIBBONBAR_TAB_RIGHT_DOWN, 1)
        EVT_RIBBONBAR_TAB_RIGHT_UP    = wx.PyEventBinder(wxEVT_RIBBONBAR_TAB_RIGHT_UP, 1)
        EVT_RIBBONBAR_TAB_LEFT_DCLICK = wx.PyEventBinder(wxEVT_RIBBONBAR_TAB_LEFT_DCLICK, 1)
        EVT_RIBBONBAR_TOGGLED         = wx.PyEventBinder(wxEVT_RIBBONBAR_TOGGLED, 1)
        EVT_RIBBONBAR_HELP_CLICK      = wx.PyEventBinder(wxEVT_RIBBONBAR_HELP_CLICK, 1)
        """)


    module.addItem(
        tools.wxArrayWrapperTemplate('wxRibbonPageTabInfoArray',
                                     'wxRibbonPageTabInfo',
                                     module))


    #-----------------------------------------------------------------
    tools.doCommonTweaks(module)
    tools.runGenerators(module)


#---------------------------------------------------------------------------
if __name__ == '__main__':
    run()




#---------------------------------------------------------------------------
# Name:        etg/mousestate.py
# Author:      Robin Dunn
#
# Created:     15-Nov-2010
# Copyright:   (c) 2010-2020 by Total Control Software
# License:     wxWindows License
#---------------------------------------------------------------------------

import etgtools
import etgtools.tweaker_tools as tools

PACKAGE   = "wx"
MODULE    = "_core"
NAME      = "mousestate"   # Base name of the file to generate to for this script
DOCSTRING = ""

# The classes and/or the basename of the Doxygen XML files to be processed by
# this script.
ITEMS  = [ 'wxMouseState' ]

#---------------------------------------------------------------------------

def run():
    # Parse the XML file(s) building a collection of Extractor objects
    module = etgtools.ModuleDef(PACKAGE, MODULE, NAME, DOCSTRING)
    etgtools.parseDoxyXML(module, ITEMS)

    #-----------------------------------------------------------------
    # Tweak the parsed meta objects in the module object as needed for
    # customizing the generated code and docstrings.


    c = module.find('wxMouseState')
    c.find('GetPosition').findOverload('int *x').ignore()

    c.addProperty("x GetX SetX")
    c.addProperty("y GetY SetY")
    c.addProperty("X GetX SetX")
    c.addProperty("Y GetY SetY")
    c.addProperty("leftIsDown LeftIsDown SetLeftDown")
    c.addProperty("middleIsDown MiddleIsDown SetMiddleDown")
    c.addProperty("rightIsDown RightIsDown SetRightDown")
    c.addProperty("aux1IsDown Aux1IsDown SetAux1Down")
    c.addProperty("aux2IsDown Aux2IsDown SetAux2Down")
    c.addProperty("Position GetPosition SetPosition")


    #-----------------------------------------------------------------
    tools.doCommonTweaks(module)
    tools.runGenerators(module)


#---------------------------------------------------------------------------
if __name__ == '__main__':
    run()




#---------------------------------------------------------------------------
# Name:        etg/notifmsg.py
# Author:      Robin Dunn
#
# Created:     21-May-2012
# Copyright:   (c) 2012-2020 by Total Control Software
# License:     wxWindows License
#---------------------------------------------------------------------------

import etgtools
import etgtools.tweaker_tools as tools

PACKAGE   = "wx"
MODULE    = "_adv"
NAME      = "notifmsg"   # Base name of the file to generate to for this script
DOCSTRING = ""

# The classes and/or the basename of the Doxygen XML files to be processed by
# this script.
ITEMS  = [ "wxNotificationMessage",
           ]

#---------------------------------------------------------------------------

def run():
    # Parse the XML file(s) building a collection of Extractor objects
    module = etgtools.ModuleDef(PACKAGE, MODULE, NAME, DOCSTRING)
    etgtools.parseDoxyXML(module, ITEMS)

    #-----------------------------------------------------------------
    # Tweak the parsed meta objects in the module object as needed for
    # customizing the generated code and docstrings.

    c = module.find('wxNotificationMessage')
    assert isinstance(c, etgtools.ClassDef)
    c.mustHaveApp()

    # take care of some methods only available on MSW
    c.find('UseTaskBarIcon').setCppCode("""\
        #ifdef __WXMSW__
            return wxNotificationMessage::UseTaskBarIcon(icon);
        #else
            wxPyRaiseNotImplemented();
            return NULL;
        #endif
        """)

    c.find('MSWUseToasts').setCppCode("""\
        #ifdef __WXMSW__
            return wxNotificationMessage::MSWUseToasts(*shortcutPath, *appId);
        #else
            wxPyRaiseNotImplemented();
            return false;
        #endif
        """)

    # TODO: Also add wxGenericNotificationMessage

    #-----------------------------------------------------------------
    tools.doCommonTweaks(module)
    tools.runGenerators(module)


#---------------------------------------------------------------------------
if __name__ == '__main__':
    run()




import unittest
import wx
import six
from unittests import wtc

#---------------------------------------------------------------------------



class String(unittest.TestCase):

    if hasattr(wx, 'testStringTypemap'):
        if not six.PY3:
            def test_StringTypemapsPy2(self):
                utf  = '\xc3\xa9l\xc3\xa9phant'     # utf-8 string
                uni  = utf.decode('utf-8')          # convert to unicode
                iso  = uni.encode('iso-8859-1')     # make a string with a different encoding


                # wx.testStringTypemap() will accept a parameter that
                # is a Unicode object or an 'ascii' or 'utf-8' string object,
                # which will then be converted to a wxString. The return
                # value is a Unicode object that has been converted from a
                # wxString that is a copy of the wxString created for the
                # parameter.

                # ascii
                result = wx.testStringTypemap('hello')
                self.assertTrue(type(result) == unicode)
                self.assertTrue(result == unicode('hello'))

                # unicode should pass through unmodified
                result = wx.testStringTypemap(uni)
                self.assertTrue(result == uni)

                # utf-8 is converted
                result = wx.testStringTypemap(utf)
                self.assertTrue(result == uni)

                # can't auto-convert this
                with self.assertRaises(UnicodeDecodeError):
                    result = wx.testStringTypemap(iso)

                # utf-16-be
                val = "\x00\xe9\x00l\x00\xe9\x00p\x00h\x00a\x00n\x00t"
                with self.assertRaises(UnicodeDecodeError):
                    result = wx.testStringTypemap(val)
                result = wx.testStringTypemap( val.decode('utf-16-be'))
                self.assertTrue(result == uni)

                # utf-32-be
                val = "\x00\x00\x00\xe9\x00\x00\x00l\x00\x00\x00\xe9\x00\x00\x00p\x00\x00\x00h\x00\x00\x00a\x00\x00\x00n\x00\x00\x00t"
                with self.assertRaises(UnicodeDecodeError):
                    result = wx.testStringTypemap(val)
                result = wx.testStringTypemap(val.decode('utf-32-be'))
                self.assertTrue(result == uni)

                # utf-8 with BOM
                #val = "\xef\xbb\xbfHello"
                #result = wx.testStringTypemap(val)
                #self.assertTrue(result == u'\ufeffHello')

        else:
            def test_StringTypemapsPy3(self):
                utf  = b'\xc3\xa9l\xc3\xa9phant'    # utf-8 bytes
                uni  = utf.decode('utf-8')          # convert to unicode
                iso  = uni.encode('iso-8859-1')     # make a string with a different encoding

                # ascii
                result = wx.testStringTypemap(b'hello')
                self.assertTrue(type(result) == str)
                self.assertTrue(result == 'hello')

                # unicode should pass through unmodified
                result = wx.testStringTypemap(uni)
                self.assertTrue(result == uni)

                # utf-8 is converted
                result = wx.testStringTypemap(utf)
                self.assertTrue(result == uni)

                # can't auto-convert this
                with self.assertRaises(UnicodeDecodeError):
                    result = wx.testStringTypemap(iso)


#---------------------------------------------------------------------------


if __name__ == '__main__':
    unittest.main()



import sys
import os
import re

if sys.version_info < (3,):
    from StringIO import StringIO
else:
    from io import StringIO

from inspect import getmro, getclasstree, getdoc, getcomments

from .utilities import makeSummary, chopDescription, writeSphinxOutput, PickleFile
from .utilities import findControlImages, formatExternalLink, isPython3
from .constants import object_types, MODULE_TO_ICON, DOXY_2_REST, SPHINXROOT
from . import templates

EPYDOC_PATTERN = re.compile(r'\S+{\S+}', re.DOTALL)

if sys.version_info < (3,):
    reload(sys)
    sys.setdefaultencoding('utf-8')


def make_class_tree(tree):

    class_tree = []

    if isinstance(tree, list):
        for node in tree:
            class_tree.append(make_class_tree(node))
    else:
        name = tree[0].__name__
        class_tree.append(name)

    return class_tree


def generic_summary(libraryItem, stream):

    write_toc = True
    add_tilde = [True, True]

    if libraryItem.kind in [object_types.LIBRARY, object_types.PACKAGE]:
        list1 = libraryItem.GetItemByKind(object_types.PACKAGE)
        list2 = libraryItem.GetItemByKind(object_types.PY_MODULE, object_types.PYW_MODULE)
        list3 = libraryItem.GetItemByKind(object_types.FUNCTION)
        list4 = libraryItem.GetItemByKind(object_types.KLASS, recurse=True)

        all_lists = [list1, list2, list3, list4]
        templ = [ templates.TEMPLATE_PACKAGE_SUMMARY,
                  templates.TEMPLATE_MODULE_SUMMARY,
                  templates.TEMPLATE_STD_FUNCTION_SUMMARY,
                  templates.TEMPLATE_STD_CLASS_SUMMARY
                  ]
        refs = ['mod', 'mod', 'func', 'ref']
        add_tilde = [True, True, True, True]

    elif libraryItem.kind in range(object_types.PY_MODULE, object_types.PYW_MODULE+1):
        list1 = libraryItem.GetItemByKind(object_types.FUNCTION)
        list2 = libraryItem.GetItemByKind(object_types.KLASS, recurse=True)

        all_lists = [list1, list2]
        templ = [templates.TEMPLATE_STD_FUNCTION_SUMMARY, templates.TEMPLATE_STD_CLASS_SUMMARY]
        refs = ['func', 'ref']
        add_tilde = [True, True]

    elif libraryItem.kind == object_types.KLASS:
        write_toc = False
        list1 = libraryItem.GetItemByKind(object_types.METHOD, object_types.BUILTIN_FUNCTION)
        list2 = libraryItem.GetItemByKind(object_types.PROPERTY)

        all_lists = [list1, list2]
        templ = [templates.TEMPLATE_METHOD_SUMMARY, templates.TEMPLATE_PROPERTY_SUMMARY]
        refs = ['meth', 'attr']
        add_tilde = [True, True]

    else:
        raise Exception('Invalid library item: %s'%libraryItem.GetShortName())

    toctree = ''

    for index, sub_list in enumerate(all_lists):
        table = []
        for item in sub_list:

            if item.is_redundant:
                continue

            item_docs = replaceWxDot(item.docs)
            item_docs = killEpydoc(item, item_docs)
            docs = chopDescription(item_docs)
            table.append((item.name, docs))

            if item.kind != object_types.FUNCTION:
                toctree += '   %s\n'%item.name

        if table:
            summary = makeSummary(libraryItem.name, table, templ[index], refs[index], add_tilde[index])
            stream.write(summary)

    if toctree and write_toc:
        stream.write(templates.TEMPLATE_TOCTREE%toctree)
        stream.write('\n\n')


def makeSphinxFile(name):

    return os.path.join(os.getcwd(), 'docs', 'sphinx', '%s.txt' % name)


def replaceWxDot(text):

    # Masked is funny...
    text = text.replace('</LI>', '')

    space_added = False
    for old, new in DOXY_2_REST:

        if old not in text:
            continue

        if new in [':keyword', ':param']:
            if not space_added:
                space_added = True
                new_with_newline = '\n%s' % new
                text = text.replace(old, new_with_newline, 1)

        text = text.replace(old, new)

    lines = text.splitlines(True)
    newtext = ''

    for line in lines:
        if '@section' not in line:
            newtext += line
            continue

        # Extract the section header
        splitted = line.split()
        header = ' '.join(splitted[2:])
        header = header.strip()

        newtext += header + '\n'
        newtext += '-'*len(header) + '\n\n'

    # Try and replace True with ``True`` and False with ``False``
    # ``None`` gives trouble sometimes...

    for keyword in ['True', 'False']:
        newtext = re.sub(r'\s%s\s'%keyword, ' ``%s`` '%keyword, newtext)

    return newtext


def GetTopLevelParent(klass):

    parent = klass.parent

    if not parent:
        return klass

    parents = [parent]

    while parent:
        parent = parent.parent
        parents.append(parent)

    return parents[-2]


def findInHierarchy(klass, newlink):

    library = GetTopLevelParent(klass)
    return library.FindItem(newlink)


def findBestLink(klass, newlink):

    parent_class = klass.parent

    if klass.kind in range(object_types.FUNCTION, object_types.INSTANCE_METHOD):
        if parent_class.GetShortName() == newlink:
            return ':class:`%s`'%newlink
        else:
            child_names = [sub.GetShortName() for sub in parent_class.children]
            if newlink in child_names:
                index = child_names.index(newlink)
                child = parent_class.children[index]

                if child.kind in range(object_types.PACKAGE, object_types.PYW_MODULE+1):
                    return ':mod:`~%s`'%child.name
                elif child.kind in range(object_types.FUNCTION, object_types.INSTANCE_METHOD):
                    return ':meth:`~%s`'%child.name
                elif child.kind == object_types.KLASS:
                    return ':class:`~%s`'%child.name
                else:
                    return ':attr:`~%s`'%child.name

    full_loop = findInHierarchy(klass, newlink)

    if full_loop:
        return full_loop

    return ':ref:`%s`'%newlink


def killEpydoc(klass, newtext):

    epydocs = re.findall(EPYDOC_PATTERN, newtext)

    if not epydocs:
        return newtext

    newepydocs = epydocs[:]

    for item in epydocs:
        if '#{' in item:
            # this is for masked stuff
            newepydocs.remove(item)

    if not newepydocs:
        return newtext

    for regex in newepydocs:

        start = regex.index('{')
        end   = regex.index('}')

        if 'U{' in regex:
            # Simple link, leave it as it is
            newlink = regex[start+1:end]

        elif 'C{' in regex:
            # It's an inclined text, but we can attach some
            # class reference to it
            newlink = regex[start+1:end]

            if 'wx.' in regex or 'wx' in regex:
                newlink = ':class:`%s`' % newlink.strip()
            else:
                newlink = '`%s`' % newlink

        elif 'I{' in regex:
            # It's an inclined text
            newlink = regex[start+1:end]
            newlink = ' `%s` ' % newlink

        elif 'L{' in regex:
            # Some kind of link, but we can't figure it out
            # very easily from here... just use :ref:
            newlink = regex[start+1:end]

            if newlink.upper() == newlink:
                # Use double backticks
                newlink = '``%s``' % newlink
            else:
                # Try and reference it
                bestlink = findBestLink(klass, newlink)
                if bestlink:
                    newlink = bestlink

        else:
            # Something else, don't bother for the moment
            continue

        newtext = newtext.replace(regex, newlink)

    return newtext


class ParentBase(object):

    def __init__(self, name, kind):

        self.name = name
        self.kind = kind

        self.docs = ''
        self.comments = ''

        self.is_redundant = False

        self.children = []


    def Add(self, klass):

        if 'lambda' in klass.name:
            return

        for child in self.children:
            if child.name == klass.name:
                return

        klass.parent = self
        self.children.append(klass)


    def Save(self):

        if self.GetShortName().startswith('__test') or '.extern.' in self.name:
            self.is_redundant = True

        self.children = sorted(self.children, key=lambda k: (getattr(k, 'order'), getattr(k, 'name').lower()))

        if self.docs is None:
            self.docs = ''

        if self.comments is None or not self.comments.strip():
            self.comments = ''

        for child in self.children:
            child.Save()


    def GetImage(self):

        return self.kind


    def GetName(self):

        return self.name


    def GetShortName(self):

        return self.name.split('.')[-1]


    def GetObject(self):

        return self.obj_type


    def GetChildren(self):

        return self.children


    def GetChildrenCount(self, recursively=True):
        """
        Gets the number of children of this item.

        :param bool `recursively`: if ``True``, returns the total number of descendants,
         otherwise only one level of children is counted.
        """

        count = len(self.children)

        if not recursively:
            return count

        total = count

        for n in range(count):
            total += self.children[n].GetChildrenCount()

        return total


    def GetKindCount(self, minObj, maxObj=None):

        if maxObj is None:
            maxObj = minObj

        count = 0
        for child in self.children:
            if minObj <= child.kind <= maxObj:
                count += 1

        return count


    def GetItemByKind(self, minObj, maxObj=None, recurse=False):

        if maxObj is None:
            maxObj = minObj

        items = []
        for child in self.children:
            if minObj <= child.kind <= maxObj:
                items.append(child)

                if recurse:
                    items = items + child.GetItemByKind(minObj, maxObj, recurse)

        return items


    def ToRest(self, class_summary):

        pass


class Library(ParentBase):

    def __init__(self, name):

        ParentBase.__init__(self, name, object_types.LIBRARY)

        self.parent = None
        self.filename = ''
        self.order = 0
        self.obj_type = 'Library'
        self.python_version = ''

        self.sphinx_file = makeSphinxFile(name)
        self.base_name = name


    def GetShortName(self):

        return self.name


    def Walk(self, obj, class_summary):

        if obj == self:
            obj.ToRest(class_summary)

        # must have at least root folder
        children = obj.GetChildren()

        if not children:
            return

        # check each name
        for child in children:

            if child.is_redundant:
                continue

            child.ToRest(class_summary)

            # recursively scan other folders, appending results
            self.Walk(child, class_summary)


    def FindItem(self, newlink, obj=None):

        if obj is None:
            obj = self

        # must have at least root folder
        children = obj.GetChildren()
        bestlink = ''

        if not children:
            return bestlink

        # check each name
        for child in children:

            if child.is_redundant:
                continue

            parts = child.name.split('.')
            dotname = '.'.join(parts[-2:])
            if child.name.endswith(newlink) and (child.GetShortName() == newlink or dotname == newlink):
                if child.kind in range(object_types.PACKAGE, object_types.PYW_MODULE+1):
                    return ':mod:`~%s`'%child.name
                elif child.kind in range(object_types.FUNCTION, object_types.INSTANCE_METHOD+1):
                    return ':meth:`~%s`'%child.name
                elif child.kind == object_types.KLASS:
                    return ':class:`~%s`'%child.name
                else:
                    return ':attr:`~%s`'%child.name

            bestlink = self.FindItem(newlink, child)

            if bestlink:
                return bestlink

        return bestlink


    def GetPythonVersion(self):

        return self.python_version


    def ToRest(self, class_summary):

        print('\n\nReST-ifying %s...\n\n' % self.base_name)
        stream = StringIO()

        header = templates.TEMPLATE_DESCRIPTION%(self.base_name, self.base_name)
        stream.write(header)

        newtext = replaceWxDot(self.docs)
        newtext = killEpydoc(self, newtext)

        stream.write(newtext + '\n\n')

        generic_summary(self, stream)
        writeSphinxOutput(stream, self.sphinx_file)


    def ClassesToPickle(self, obj, class_dict):

        # must have at least root folder
        children = obj.GetChildren()

        if not children:
            return class_dict

        # check each name
        for child in children:
            if child.kind == object_types.KLASS:
                if child.is_redundant:
                    continue

                class_dict[child.name] = (child.method_list, child.bases, chopDescription(child.docs))

            # recursively scan other folders, appending results
            class_dict = self.ClassesToPickle(child, class_dict)

        return class_dict


    def Save(self):
        ParentBase.Save(self)

        class_dict = {}
        class_dict = self.ClassesToPickle(self, class_dict)

        pickle_file = os.path.join(SPHINXROOT, 'class_summary.pkl')
        with PickleFile(pickle_file) as pf:
            pf.items.update(class_dict)


class Module(ParentBase):

    def __init__(self, name, kind):

        ParentBase.__init__(self, name, kind)

        self.filename = ''
        self.sphinx_file = makeSphinxFile(name)

        if kind == object_types.PACKAGE:
            self.obj_type = 'Package'
            self.order = kind
            return

        self.order = object_types.PY_MODULE

        for dummy, icon, description in MODULE_TO_ICON:
            if icon == kind:
                self.obj_type = description
                break

        self.inheritance_diagram = None


    def ToRest(self, class_summary):

        if self.is_redundant:
            return

        stream = StringIO()

        label = 'Module'
        if self.kind == object_types.PACKAGE:
            label = 'Package'

        stream.write('.. module:: %s\n\n' % self.name)
        stream.write('.. currentmodule:: %s\n\n' % self.name)
        stream.write('.. highlight:: python\n\n')

        header = templates.TEMPLATE_DESCRIPTION % (self.name, self.name)

        stream.write(header)

        newtext = replaceWxDot(self.docs)
        newtext = killEpydoc(self, newtext)

        stream.write(newtext + '\n\n')

        spacer = '   '*self.name.count('.')

        # IMPORTANT!!
        # Remove this line to get back the inheritance diagram for a module
        #
        self.inheritance_diagram = None

        if self.kind != object_types.PACKAGE:
            print(('%s - %s (module)'%(spacer, self.name)))
            if self.inheritance_diagram:
                png, map = self.inheritance_diagram.makeInheritanceDiagram(class_summary)
                short_name = self.GetShortName()
                image_desc = templates.TEMPLATE_INHERITANCE % ('module', short_name, png, short_name, map)
                stream.write(image_desc)
        else:
            print('%s - %s (package)' % (spacer, self.name))

        generic_summary(self, stream)

        functions = self.GetItemByKind(object_types.FUNCTION)

        count = 0
        for fun in functions:
            if not fun.is_redundant:
                count = 1
                break

        if count > 0:
            stream.write('\n\nFunctions\n------------\n\n')

        for fun in functions:
            if fun.is_redundant:
                continue
            fun.Write(stream)

        writeSphinxOutput(stream, self.sphinx_file)


    def Save(self):

        ParentBase.Save(self)

        if self.GetShortName().startswith('__test') or '.extern.' in self.name:
            self.is_redundant = True


class Class(ParentBase):

    def __init__(self, name, obj):

        ParentBase.__init__(self, name, object_types.KLASS)

        try:
            subs = obj.__subclasses__()
        except (AttributeError, TypeError):
            subs = []

        sups = list(obj.__bases__)

        sortedSubClasses = []
        sortedSupClasses = []

        for item in sups:
            item = repr(item)

            sup = item.replace('<class ', '').replace('>', '').replace('<type ', '')
            sup = sup.strip().replace('"', '').replace("'", '')
            if ' at ' in sup:
                sup = sup[0:sup.index(' at ')].strip()

            if sup.startswith('wx._'):
                # take out the '_core' in things like 'wx._core.Control'
                parts = sup.split('.')
                if parts[1] == '_core':
                    del parts[1]
                else:
                    # or just the '_' otherwise
                    parts[1] = parts[1][1:]
                sup = '.'.join(parts)

            sortedSupClasses.append(sup)

        sortedSupClasses.sort()

        for s in subs:
            s = repr(s)

            if "'" in s:
                start = s.index("'")
                end = s.rindex("'")
                cls = s[start+1:end]
            else:
                cls = s

            sortedSubClasses.append(cls)

        sortedSubClasses.sort()

        if len(sortedSubClasses) == 1 and sortedSubClasses[0] == 'object':
            sortedSubClasses = []

        if len(sortedSupClasses) == 1 and sortedSupClasses[0] == 'object':
            sortedSupClasses = []

        self.class_tree = make_class_tree(getclasstree(getmro(obj)))

        self.subClasses = sortedSubClasses
        self.superClasses = sortedSupClasses

        self.signature = ''
        self.inheritance_diagram = None

        self.order = 3
        self.obj_type = 'Class'
        self.sphinx_file = makeSphinxFile(name)


    def ToRest(self, class_summary):

        if self.is_redundant:
            return

        stream = StringIO()

        parts = self.name.split('.')
        current_module = '.'.join(parts[0:-1])

        stream.write('.. currentmodule:: %s\n\n' % current_module)
        stream.write('.. highlight:: python\n\n')

        class_docs = replaceWxDot(self.docs)
        class_docs = killEpydoc(self, class_docs)

        header = templates.TEMPLATE_DESCRIPTION % (self.name, self.name)
        stream.write(header)
        stream.write(class_docs + '\n\n')

        if self.inheritance_diagram:
            png, map = self.inheritance_diagram.makeInheritanceDiagram(class_summary)
            short_name = self.GetShortName()
            image_desc = templates.TEMPLATE_INHERITANCE % ('class', short_name, png, short_name, map)
            stream.write(image_desc)

        appearance = findControlImages(self.name.lower())
        if appearance:
            appearance_desc = templates.TEMPLATE_APPEARANCE % tuple(appearance)
            stream.write(appearance_desc + '\n\n')

        if self.subClasses:
            subs = [formatExternalLink(cls) for cls in self.subClasses]
            subs = ', '.join(subs)
            subs_desc = templates.TEMPLATE_SUBCLASSES % subs
            stream.write(subs_desc)

        if self.superClasses:
            sups = [formatExternalLink(cls) for cls in self.superClasses]
            sups = ', '.join(sups)
            sups_desc = templates.TEMPLATE_SUPERCLASSES % sups
            stream.write(sups_desc)

        generic_summary(self, stream)

        stream.write(templates.TEMPLATE_API)
        stream.write('\n.. class:: %s\n\n'%self.signature)

        docs = ''
        for line in class_docs.splitlines(True):
            docs += ' '*3 + line

        stream.write(docs + '\n\n')

        methods = self.GetItemByKind(object_types.METHOD, object_types.BUILTIN_FUNCTION)
        properties = self.GetItemByKind(object_types.PROPERTY)

        for meth in methods:
            meth.Write(stream)

        for prop in properties:
            prop.Write(stream, short_name)

        writeSphinxOutput(stream, self.sphinx_file)
        self.bases = self.superClasses


    def Save(self):

        ParentBase.Save(self)
        pop = -1

        for index, child in enumerate(self.children):
            name = child.GetShortName()
            if name == '__init__':
                pop = index
                break

        if pop >= 0:
            init = self.children.pop(pop)
            self.children.insert(0, init)

        #self.signature = self.signature.replace('wx.', '')
        self.signature = self.signature.rstrip(':').lstrip('class ')

        if ' def __init__' in self.signature:
            index = self.signature.index(' def __init__')
            self.signature = self.signature[0:index]

        self.signature = self.signature.strip()

        # if len(self.signature) < 2: # ???
        #     self.is_redundant = True

        if self.GetShortName().startswith('__test') or '.extern.' in self.name:
            self.is_redundant = True

        if self.is_redundant:
            return

        methods = self.GetItemByKind(object_types.METHOD, object_types.BUILTIN_FUNCTION)
        method_list = []

        for meth in methods:
            if not meth.is_redundant:
                method_list.append(meth.GetShortName())

        self.method_list = method_list
        self.bases = self.superClasses


class ChildrenBase(object):

    def __init__(self, name, kind):

        self.name = name
        self.kind = kind

        self.order = 4

        self.docs = ''
        self.comments = ''

        self.is_redundant = False

##        self.id = NewId()


    def GetImage(self):

        return self.kind


    def GetName(self):

        return self.name


    def GetShortName(self):

        return self.name.split('.')[-1]


    def GetChildren(self):

        return []


    def GetChildrenCount(self, recursively=True):

        return 0


    def GetObject(self):

        return self.obj_type


    def Save(self):

        if self.docs is None:
            self.docs = ''

        if self.comments is None or not self.comments.strip():
            self.comments = ''


    def ToRest(self, class_summary):

        pass


class Method(ChildrenBase):

    def __init__(self, name, kind):

        ChildrenBase.__init__(self, name, kind)

        self.order = 5

        self.arguments = []
        self.signature = ''

        self.obj_type = 'Method/Function'


    def Save(self):

        ChildrenBase.Save(self)

        newargs = []
        if self.arguments and any(self.arguments[0]):
            for name, repr_val, eval_val in self.arguments:
                repr_val = (repr_val is not None and [repr_val] or [''])[0]
                eval_val = (eval_val is not None and [eval_val] or [''])[0]
                newargs.append((name, repr_val, eval_val))

        self.arguments = newargs
        self.signature = self.signature.rstrip(':').lstrip()

        if self.signature.startswith('def '):
            self.signature = self.signature[4:]

        if '@staticmethod' in self.signature:
            self.kind = object_types.STATIC_METHOD
        elif '@classmethod' in self.signature:
            self.kind = object_types.CLASS_METHOD

        if ' def ' in self.signature:
            index = self.signature.index(' def ')
            self.signature = self.signature[index+5:].strip()

        if '*' in self.signature:
            self.signature = self.signature.replace('*', r'\*')

        if not self.signature.strip():
            # if there is no signature, then check if the first line of
            # docstring looks like it might be it
            lines = self.docs.split('\n')
            first = lines[0]
            rest = '\n'.join(lines[1:]) if len(lines) > 1 else ''
            sig_start = self.GetShortName() + '('
            if sig_start in first:
                self.signature = first[first.find(sig_start):]
                self.docs = rest.strip()

        # if not self.signature.strip():      # ???
        #     self.is_redundant = True


    def Write(self, stream):

        if self.is_redundant:
            return

        if self.kind == object_types.FUNCTION:
            stream.write('.. function:: %s\n\n'%self.signature)
            indent = 3*' '
        else:
            if self.kind == object_types.STATIC_METHOD:
                stream.write('   .. staticmethod:: %s\n\n'%self.signature)
            elif self.kind == object_types.CLASS_METHOD:
                stream.write('   .. classmethod:: %s\n\n'%self.signature)
            else:
                stream.write('   .. method:: %s\n\n'%self.signature)
            indent = 6*' '

        if not self.docs.strip():
            stream.write('\n')
            return

        text = ''
        newdocs = replaceWxDot(self.docs)

        for line in newdocs.splitlines(True):
            text += indent + line

        text = killEpydoc(self, text)
        text += '\n\n\n'
        stream.write(text)


class Property(ChildrenBase):

    def __init__(self, name, item):

        ChildrenBase.__init__(self, name, object_types.PROPERTY)

        self.getter = self.setter = self.deleter = ''

        # is it a real property?
        if isinstance(item, property):
            try:
                if item.fget:
                    self.getter = item.fget.__name__
                if item.fset:
                    self.setter = item.fset.__name__
                if item.fdel:
                    self.deleter = item.fdel.__name__
            except AttributeError:
                # Thank you for screwing it up, Cython...
                if item.fget:
                    self.getter = item.fget.__class__.__name__
                if item.fset:
                    self.setter = item.fset.__class__.__name__
                if item.fdel:
                    self.deleter = item.fdel.__class__.__name__

        self.docs = getdoc(item)
        self.comments = getcomments(item)

        self.obj_type = 'Property'
        self.order = 6


    def Write(self, stream, class_name):

        if self.is_redundant:
            return

        docs = self.docs
        if not docs:
            for item in [self.setter, self.getter, self.deleter]:
                if item and 'lambda' not in item and not item.startswith('_'):
                    if docs:
                        docs += ', :meth:`~%s.%s` ' % (class_name, item)
                    else:
                        docs += ':meth:`~%s.%s` ' % (class_name, item)

            if docs:
                docs = 'See %s' % docs

        if docs:
            stream.write('   .. attribute:: %s\n\n' % self.GetShortName())
            docs = '\n      '.join(docs.splitlines())
            stream.write('      %s\n\n\n' % docs)


class Attribute(ChildrenBase):

    def __init__(self, name, specs, value):

        if isPython3():
            specs = str(specs)
        else:
            specs = unicode(specs)

        start, end = specs.find("'"), specs.rfind("'")
        specs = specs[start+1:end]

        strValue = repr(value)
        uspecs = specs.upper()

        try:
            kind = getattr(object_types, uspecs)
        except AttributeError:
            try:
                uspecs = uspecs + 'TYPE'
                kind = getattr(object_types, uspecs)
            except AttributeError:
                kind = object_types.UNKNOWNTYPE

        try:
            reprValue = repr(value.__class__)
        except (NameError, AttributeError):
            reprValue = ''

        if 'class' in strValue or 'class' in reprValue:
            kind = object_types.INSTANCETYPE

        ChildrenBase.__init__(self, name, kind)

        self.value = strValue
        self.specs = specs

        try:
            self.docs = getdoc(value)
        except (NameError, AttributeError):
            self.docs = ''

        self.obj_type = 'Attribute'
        self.order = 7


    def ToRest(self, class_summary):

        pass




import unittest
from unittests import wtc
import wx
import sys

#---------------------------------------------------------------------------

class fontenum_Tests(wtc.WidgetTestCase):

    @unittest.skipIf(sys.platform == 'darwin', 'EnumerateEncodings not implemented on Mac')
    def test_fontenum1(self):
        enc = wx.FontEnumerator.GetEncodings()
        self.assertTrue(isinstance(enc, list))
        self.assertTrue(len(enc) > 0)

    def test_fontenum2(self):
        faces = wx.FontEnumerator.GetFacenames()
        self.assertTrue(isinstance(faces, list))
        self.assertTrue(len(faces) > 0)

        fw_faces = wx.FontEnumerator.GetFacenames(fixedWidthOnly=True)
        self.assertTrue(len(fw_faces) < len(faces))


    def test_fontenum3(self):
        class MyFontEnumerator(wx.FontEnumerator):
            def __init__(self):
                wx.FontEnumerator.__init__(self)
                self.my_faces = list()
                self.my_enc = list()

            def OnFacename(self, name):
                self.my_faces.append(name)
                return len(self.my_faces) < 5  # quit when we've got 5
            def OnFontEncoding(self, facename, encoding):
                self.my_enc.append( (facename, encoding) )
                return len(self.my_enc) < 5  # quit when we've got 5

        mfe = MyFontEnumerator()
        mfe.EnumerateFacenames()
        self.assertTrue(len(mfe.my_faces) > 0 and len(mfe.my_faces) <= 5)

        if sys.platform != 'darwin':
            mfe.EnumerateEncodings()
            self.assertTrue(len(mfe.my_enc) > 0 and len(mfe.my_enc) <= 5)

#---------------------------------------------------------------------------

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# This file is used for the HtmlWindow demo.

import  sys

import  wx
import  wx.html as  html

#----------------------------------------------------------------------

class TestPanel(wx.Panel):
    def __init__(self, parent, id=-1, size=wx.DefaultSize, bgcolor=None):
        wx.Panel.__init__(self, parent, id, size=size)

        if bgcolor:
            self.SetBackgroundColour(bgcolor)

        wx.StaticText(self, -1, 'Name:', (10, 10))
        wx.StaticText(self, -1, 'Email:', (10, 40))

        self.name  = wx.TextCtrl(self, -1, '', (50, 10), (100, -1))
        self.email = wx.TextCtrl(self, -1, '', (50, 40), (100, -1))

        wx.Button(self, -1, 'Okay', (50, 70))
        self.Bind(wx.EVT_BUTTON, self.OnButton)


    def OnButton(self, event):
        name = self.name.GetValue()
        email = self.email.GetValue()
        dlg = wx.MessageDialog(
                self, 'You entered:\n    %s\n    %s' % (name, email),
                'Results', style = wx.OK | wx.ICON_INFORMATION
                )

        dlg.ShowModal()
        dlg.Destroy()



#----------------------------------------------------------------------

class TestHtmlPanel(wx.Panel):
    def __init__(self, parent, id=-1, size=wx.DefaultSize):

        import About

        wx.Panel.__init__(self, parent, id, size=size)
        self.html = html.HtmlWindow(self, -1, (5,5), (400, 350))
        py_version = sys.version.split()[0]
        self.html.SetPage(About.MyAboutBox.text %
                          (wx.VERSION_STRING,
                           ", ".join(wx.PlatformInfo[1:]),
                           py_version))
        ir = self.html.GetInternalRepresentation()
        self.html.SetSize( (ir.GetWidth()+5, ir.GetHeight()+5) )
        self.Fit()

#----------------------------------------------------------------------

def runTest(frame, nb, log):
    win = TestHtmlPanel(frame)
    return win

#----------------------------------------------------------------------

if __name__ == '__main__':
    import sys,os
    import run
    run.main(['', os.path.basename(sys.argv[0])] + sys.argv[1:])




#!/usr/bin/env python

import wx
import os

#----------------------------------------------------------------------
class TestPanel(wx.Panel):
    def __init__(self, parent, log):
        self.log = log
        wx.Panel.__init__(self, parent)

        # Event Handlers
        self.Bind(wx.EVT_PAINT, self.OnPaint)


    def OnPaint(self, event):
        dc = wx.GCDC(wx.PaintDC(self))
        #dc = wx.PaintDC(self)
        render = wx.RendererNative.Get()

        # Setup Brushes
        dc.SetBrush(wx.BLACK_BRUSH)
        dc.SetTextForeground(wx.BLACK)
        dc.SetFont(wx.NORMAL_FONT)

        # The below code will use RendererNative to draw controls in
        # various states. The wx.CONTROL_* flags are used to tell the
        # Renderer which state to draw the control in.

        # Draw some checkboxes
        cb_lbl = "DrawCheckBoxes:"
        dc.DrawText(cb_lbl, 15, 15)
        render.DrawCheckBox(self, dc, (25, 35, 16, 16), wx.CONTROL_CHECKED)
        render.DrawCheckBox(self, dc, (45, 35, 16, 16), wx.CONTROL_CHECKABLE)
        render.DrawCheckBox(self, dc, (65, 35, 16, 16))
        render.DrawCheckBox(self, dc, (85, 35, 16, 16), wx.CONTROL_CHECKED | wx.CONTROL_DISABLED)

        lbl = "DrawRadioBitmap:"
        dc.DrawText(lbl, 375, 15)
        render.DrawRadioBitmap(self, dc, (385, 35, 16, 16), wx.CONTROL_CHECKED)
        render.DrawRadioBitmap(self, dc, (405, 35, 16, 16), wx.CONTROL_CHECKABLE)
        render.DrawRadioBitmap(self, dc, (425, 35, 16, 16))
        render.DrawRadioBitmap(self, dc, (445, 35, 16, 16), wx.CONTROL_CHECKED | wx.CONTROL_DISABLED)

        # Draw ComboBoxDropButton
        xpos = self.GetTextExtent(cb_lbl)[0] + 40
        cb_lbl = "DrawComboBoxDropButton:"
        dc.DrawText(cb_lbl, xpos, 15)
        render.DrawComboBoxDropButton(self, dc, (xpos + 4, 35, 24, 24), wx.CONTROL_CURRENT)
        render.DrawComboBoxDropButton(self, dc, (xpos + 44, 35, 24, 24), wx.CONTROL_PRESSED)
        render.DrawComboBoxDropButton(self, dc, (xpos + 84, 35, 24, 24), wx.CONTROL_CURRENT | wx.CONTROL_DISABLED)
        render.DrawComboBoxDropButton(self, dc, (xpos + 124, 35, 24, 24), wx.CONTROL_PRESSED | wx.CONTROL_DISABLED)

        # Draw DropArrow
        da_lbl = "DrawDropArrow:"
        dc.DrawText(da_lbl, 15, 80)
        render.DrawDropArrow(self, dc, (15, 100, 24, 24), wx.CONTROL_CURRENT)
        render.DrawDropArrow(self, dc, (35, 100, 24, 24), wx.CONTROL_PRESSED)
        render.DrawDropArrow(self, dc, (55, 100, 24, 24), wx.CONTROL_CURRENT | wx.CONTROL_DISABLED)

        # Draw HeaderButton
        dc.DrawText("DrawHeaderButton:", xpos, 80)
        # Set some extra options for drawing
        opts = wx.HeaderButtonParams()
        hb_lbl = "HeaderButton Selected"
        opts.m_labelText = hb_lbl
        render.DrawHeaderButton(self, dc, (xpos, 100, self.GetTextExtent(hb_lbl)[0] + 30, 16),
                                wx.CONTROL_SELECTED, wx.HDR_SORT_ICON_DOWN, opts)
        hb_lbl = "HeaderButton Normal"
        opts.m_labelText = hb_lbl
        render.DrawHeaderButton(self, dc, (xpos, 125, self.GetTextExtent(hb_lbl)[0] + 30, 16),
                                sortArrow=wx.HDR_SORT_ICON_UP, params=opts)

        hb_lbl = "HeaderButton Current"
        opts.m_labelText = hb_lbl
        render.DrawHeaderButton(self, dc, (xpos, 150, self.GetTextExtent(hb_lbl)[0] + 30, 16),
                                wx.CONTROL_CURRENT, params=opts)

        # Draw ItemSelectionRect
        isr_lbl = "DrawItemSelectionRect:"
        dc.DrawText(isr_lbl, 15, 185)
        render.DrawItemSelectionRect(self, dc, (15, 205, 40, 24), wx.CONTROL_SELECTED)
        render.DrawItemSelectionRect(self, dc, (65, 205, 40, 24), wx.CONTROL_CURRENT)
        render.DrawItemSelectionRect(self, dc, (115, 205, 40, 24), wx.CONTROL_FOCUSED)

        # DrawPushButton
        pb_lbl = "DrawPushButton:"
        dc.DrawText(pb_lbl, 15, 255)
        render.DrawPushButton(self, dc, (15, 275, 45, 24), wx.CONTROL_CURRENT)
        render.DrawPushButton(self, dc, (70, 275, 45, 24), wx.CONTROL_PRESSED | wx.CONTROL_SELECTED)
        render.DrawPushButton(self, dc, (125, 275, 45, 24), wx.CONTROL_ISDEFAULT)
        render.DrawPushButton(self, dc, (180, 275, 45, 24), wx.CONTROL_CURRENT | wx.CONTROL_DISABLED)

        # DrawTreeItemButton
        ti_lbl = "DrawTreeItemButton:"
        dc.DrawText(ti_lbl, 15, 330)
        render.DrawTreeItemButton(self, dc, (15, 350, 16, 16))
        render.DrawTreeItemButton(self, dc, (45, 350, 16, 16), wx.CONTROL_EXPANDED)

        # DrawComboBox
        dc.DrawText("DrawComboBox:", 270, 185)
        render.DrawComboBox(self, dc, (270, 205, 100, 21))
        render.DrawComboBox(self, dc, (270, 230, 100, 21), wx.CONTROL_DISABLED)
        render.DrawComboBox(self, dc, (270, 255, 100, 21), wx.CONTROL_CURRENT)
        render.DrawComboBox(self, dc, (270, 280, 100, 21), wx.CONTROL_PRESSED | wx.CONTROL_SELECTED)
        render.DrawComboBox(self, dc, (270, 305, 100, 21), wx.CONTROL_FOCUSED)

        # DrawChoice
        dc.DrawText("DrawChoice:", 400, 185)
        render.DrawChoice(self, dc, (400, 205, 100, 21))
        render.DrawChoice(self, dc, (400, 230, 100, 21), wx.CONTROL_DISABLED)
        render.DrawChoice(self, dc, (400, 255, 100, 21), wx.CONTROL_CURRENT)
        render.DrawChoice(self, dc, (400, 280, 100, 21), wx.CONTROL_PRESSED | wx.CONTROL_SELECTED)
        render.DrawChoice(self, dc, (400, 305, 100, 21), wx.CONTROL_FOCUSED)

        # DrawTextCtrl
        dc.DrawText("DrawTextCtrl:", 270, 350)
        render.DrawTextCtrl(self, dc, (270, 375, 100, 21))
        render.DrawTextCtrl(self, dc, (380, 375, 100, 21), wx.CONTROL_FOCUSED)

        # DrawItemText
        render.DrawItemText(self, dc, 'DrawItemText: wx.CONTROL_ISDEFAULT', (270, 420, 300, 21), align=wx.ALIGN_LEFT|wx.ALIGN_TOP, flags=wx.CONTROL_ISDEFAULT, ellipsizeMode=wx.ELLIPSIZE_END)
        render.DrawItemText(self, dc, 'DrawItemText: wx.CONTROL_SELECTED', (270, 440, 300, 21), align=wx.ALIGN_LEFT|wx.ALIGN_TOP, flags=wx.CONTROL_SELECTED, ellipsizeMode=wx.ELLIPSIZE_END)
        render.DrawItemText(self, dc, 'DrawItemText: wx.CONTROL_DISABLED', (270, 460, 300, 21), align=wx.ALIGN_LEFT|wx.ALIGN_TOP, flags=wx.CONTROL_DISABLED, ellipsizeMode=wx.ELLIPSIZE_END)

        # DrawGauge
        dc.DrawText("DrawGauge:", 15, 380)
        render.DrawGauge(self, dc, (15, 400, 100, 21), value=20, max= 100, flags=wx.CONTROL_ISDEFAULT)
        render.DrawGauge(self, dc, (15, 430, 100, 21), value=20, max= 100, flags=wx.CONTROL_DISABLED)
        render.DrawGauge(self, dc, (15, 460, 100, 21), value=20, max= 100, flags=wx.CONTROL_CURRENT)
        render.DrawGauge(self, dc, (15, 490, 100, 21), value=20, max= 100, flags=wx.CONTROL_FOCUSED)

#----------------------------------------------------------------------

def runTest(frame, nb, log):
    win = TestPanel(nb, log)
    return win

#----------------------------------------------------------------------

overview = """<html><body>
<h2><center>wx.RendererNative</center></h2>
<p>wx.RendererNative is a class which virtualizes drawing. It abstracts the
operations of drawing controls and allows you to draw say, a button, without
caring about exactly how it is done, in a native and platform independent way.
</p>

<p>All drawing functions take some standard parameters:<p>
<ul>
<li><b>win</b>: is the window being drawn.</li>
<li><b>dc</b>: is the wxDC to draw on. Only this device context should be used
               for drawing.</li>
<li><b>rect</b>: The bounding rectangle for the element to be drawn.</li>
<li><b>flags</b>: The optional flags (none by default) which can be a
                  combination of the wx.CONTROL_XXX constants.</li>
</ul>

<p><b>Note</b>: Each drawing function restores the wxDC attributes if it
changes them, so it is safe to assume that the same pen, brush and colours
that were active before the call to this function are still in effect
after it.</p>
</body></html>
"""

#----------------------------------------------------------------------

if __name__ == '__main__':
    import sys
    import run
    run.main(['', os.path.basename(sys.argv[0])] + sys.argv[1:])



#!/usr/bin/env python

#---------------------------------------------------------------------------
# Name:        EventManager.py
# Purpose:     A module to demonstrate wx.lib.evtmgr.EventManager.
#
# Author:      Robb Shecter (robb@acm.org)
#
# Created:     16-December-2002
# Copyright:   (c) 2002-2020 by Robb Shecter (robb@acm.org) and
#              Total Control Software
# Licence:     wxWindows license
#---------------------------------------------------------------------------

import wx
import wx.lib.evtmgr as em

#----------------------------------------------------------------------

class TestPanel(wx.Panel):
    def __init__(self, parent, log):
        wx.Panel.__init__(self, parent, -1)
        self.log = log

        fsize    = self.GetFont().GetPointSize()
        f1 = wx.Font(fsize+0, wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL)
        f2 = wx.Font(fsize+2, wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD)
        f3 = wx.Font(fsize+6, wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD)

        title1 = wx.StaticText(self, -1, 'EventManager')
        title1.SetFont(f3)
        txt = """\
        This demo shows  (1) basic uses and features of the EventManager, as well
        as  (2) how it helps with a real-world task: creating independent, object-
        oriented components."""
        message0 = wx.StaticText(self, -1, txt)
        message0.SetFont(f1)

        title2 = wx.StaticText(self, -1, 'Event Listeners')
        title2.SetFont(f2)

        txt = """\
        These objects listen to motion events from the target window, using the ability
        to register one event with multiple listeners.  They also register for mouse events
        on themselves to implement toggle-button functionality."""
        message1 = wx.StaticText(self, -1, txt)
        message1.SetFont(f1)

        title3 = wx.StaticText(self, -1, 'Target Window')
        title3.SetFont(f2)

        txt = """\
        A passive window that's used as an event generator.  Move the mouse over it to
        send events to the listeners above."""
        message2 = wx.StaticText(self, -1, txt)
        message2.SetFont(f1)

        targetPanel = Tile(self, log, bgColor=wx.Colour(80,10,10), active=0)
        buttonPanel = wx.Panel(self ,-1)
        sizer       = wx.BoxSizer(wx.HORIZONTAL)
        target      = targetPanel.tile

        sizer.Add((0,0), 1)
        for factor in [0.2, 0.3, 0.4, 0.5, 0.6, 0.7]:
            sizer.Add(Tile(buttonPanel, log, factor-0.05, target), 0, wx.ALIGN_CENTER)
            sizer.Add((0,0),1)
            sizer.Add(Tile(buttonPanel, log, factor,      target), 0, wx.ALIGN_CENTER)
            sizer.Add((0,0),1)

        buttonPanel.SetSizer(sizer)
        sizer.Fit(buttonPanel)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(title1,      0, wx.ALIGN_CENTER | wx.TOP | wx.BOTTOM, 6)
        sizer.Add(message0,    0, wx.ALIGN_CENTER | wx.ALL, 6)
        sizer.Add(title2,      0, wx.ALIGN_CENTER | wx.LEFT | wx.TOP    | wx.RIGHT, 16)
        sizer.Add(message1,    0, wx.ALIGN_CENTER | wx.ALL, 6)
        sizer.Add(buttonPanel, 0, wx.EXPAND       | wx.ALL, 16)
        sizer.Add(title3,      0, wx.ALIGN_CENTER | wx.LEFT | wx.RIGHT, 16)
        sizer.Add(message2,    0, wx.ALIGN_CENTER | wx.ALL, 6)
        sizer.Add(targetPanel, 2, wx.EXPAND       | wx.LEFT | wx.BOTTOM | wx.RIGHT, 16)
        self.SetSizer(sizer)



class Tile(wx.Window):
    """
    This outer class is responsible for changing
    its border color in response to certain mouse
    events over its contained 'InnerTile'.
    """
    normal = wx.Colour(150,150,150)
    active = wx.Colour(250,245,245)
    hover  = wx.Colour(210,220,210)

    def __init__(self, parent, log, factor=1, thingToWatch=None, bgColor=None, active=1, size=(38,38), borderWidth=3):
        wx.Window.__init__(self, parent, -1, size=size, style=wx.CLIP_CHILDREN)
        self.tile = InnerTile(self, log, factor, thingToWatch, bgColor)
        self.log  = log
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(self.tile, 1, wx.EXPAND | wx.ALL, borderWidth)
        self.SetSizer(sizer)
        self.Layout()
        em.eventManager.Register(self.doLayout, wx.EVT_SIZE, self)
        self.SetBackgroundColour(Tile.normal)
        if active:
            # Register myself for mouse events over self.tile in order to
            # create typical button/hyperlink visual effects.
            em.eventManager.Register(self.setHover,  wx.EVT_ENTER_WINDOW, self.tile)
            em.eventManager.Register(self.setNormal, wx.EVT_LEAVE_WINDOW, self.tile)
            em.eventManager.Register(self.setActive, wx.EVT_LEFT_DOWN,    self.tile)
            em.eventManager.Register(self.setHover,  wx.EVT_LEFT_UP,      self.tile)


    def doLayout(self, event):
        self.Layout()


    def setHover(self, event):
        self.SetBackgroundColour(Tile.hover)
        self.Refresh()


    def setActive(self, event):
        self.SetBackgroundColour(Tile.active)
        self.Refresh()


    def setNormal(self, event):
        self.SetBackgroundColour(Tile.normal)
        self.Refresh()


class InnerTile(wx.Window):
    IDLE_COLOR  = wx.Colour( 80, 10, 10)
    START_COLOR = wx.Colour(200, 70, 50)
    FINAL_COLOR = wx.Colour( 20, 80,240)
    OFF_COLOR   = wx.Colour(185,190,185)
    # Some pre-computation.
    DELTAS            = list(map(lambda a,b: b-a, START_COLOR.Get(), FINAL_COLOR.Get()))
    START_COLOR_TUPLE = START_COLOR.Get()

    """
    This inner panel changes its color in reaction to mouse
    events over the 'thingToWatch'.
    """
    def __init__(self, parent, log, factor, thingToWatch=None, bgColor=None):
        wx.Window.__init__(self, parent, -1)
        self.SetMinSize((20,20))
        self.log=log
        if bgColor:
            self.SetBackgroundColour(bgColor)
        if thingToWatch:
            self.factor       = factor
            self.thingToWatch = thingToWatch
            self.state        = 0
            self.toggleOnOff()
            # Watch for the mouse click to enable/disable myself.
            em.eventManager.Register(self.toggleOnOff, wx.EVT_LEFT_UP, self)


    def toggleOnOff(self, event=None):
        # Implement being on or off by registering and
        # de-registering self.makeColor() from the event manager.
        if self.state:
            em.eventManager.DeregisterListener(self.makeColor)
        else:
            em.eventManager.Register(self.makeColor, wx.EVT_MOTION, self.thingToWatch)
        self.state = 1 - self.state
        self.resetColor()


    def resetColor(self, event=None):
        if self.state:
            self.setColor(InnerTile.IDLE_COLOR)
        else:
            self.setColor(InnerTile.OFF_COLOR)


    def setColor(self, color):
        self.SetBackgroundColour(color)
        self.Refresh()


    def makeColor(self, mouseEvent):
        self.makeColorFromTuple(mouseEvent.GetPosition())


    def makeColorFromTuple(self, xy):
        x, y    = xy
        MAX     = 180.0
        scaled  = min((x + y) * self.factor, MAX)  # In range [0..MAX]
        percent = scaled / MAX
        r = InnerTile.START_COLOR_TUPLE[0] + (InnerTile.DELTAS[0] * percent)
        g = InnerTile.START_COLOR_TUPLE[1] + (InnerTile.DELTAS[1] * percent)
        b = InnerTile.START_COLOR_TUPLE[2] + (InnerTile.DELTAS[2] * percent)
        self.setColor(wx.Colour(int(r), int(g), int(b)))


#----------------------------------------------------------------------

def runTest(frame, nb, log):
    win = TestPanel(nb, log)
    return win

#----------------------------------------------------------------------



overview = """<html><body>
<h2>EventManager</h2>

<p> The goal of the EventManager is to make wxWindows events more
'Pythonic' (ie. object-oriented) and easier to work with, without
impacting performance.  It offers these features:

<p>
<ul>

    <li> Allows any number of listeners to register for a single
    event.  (In addition to the standard wxPython feature of a single
    listener being able to respond to many events.)

    <li> Makes it easy to disconnect and reconnect listeners.  This
    has the effect of reducing the need for case-based branching in
    application code.

    <li> Has an object-oriented API.  Programmers register to get
    events directly from the objects that generate them, instead of
    using ID numbers.

</ul>

<h3>Usage</h3>

<p>The EventManager class has three public methods.  First get a
reference to it:

<PRE>
  from wxPython.lib.evtmgr import eventManager
</PRE>

<p>...and then invoke any of the following methods.  These methods are
'safe'; duplicate registrations or de-registrations will have no
effect.

<p><b>Registering a listener:</b>

<PRE>
  eventManager.Register(listener, event, event-source)
</PRE>


<p><b>De-registering by window:</b>

<PRE>
  eventManager.DeregisterWindow(event-source)
</PRE>


<p><b>De-registering by listener:</b>

<PRE>
  eventManager.DeregisterListener(listener)
</PRE>

<p><b>Simple Example:</b>

<PRE>
  from wxPython.lib.evtmgr import eventManager

  aButton = wxButton(somePanel, -1, 'Click me')
  eventManager.Register(self.someMethod, EVT_BUTTON, aButton)
</PRE>

<p> See the demo code as well as the documentation in the source of
<tt>wxPython.lib.evtmgr</tt> for more details.


<p>
by Robb Shecter (robb@acm.org)
</body></html>
"""



if __name__ == '__main__':
    import sys,os
    import run
    run.main(['', os.path.basename(sys.argv[0])] + sys.argv[1:])




#---------------------------------------------------------------------------
# Name:        etg/grid.py
# Author:      Robin Dunn
#
# Created:     20-Dec-2012
# Copyright:   (c) 2012-2020 by Total Control Software
# License:     wxWindows License
#---------------------------------------------------------------------------

import etgtools
import etgtools.tweaker_tools as tools

PACKAGE   = "wx"
MODULE    = "_grid"
NAME      = "grid"   # Base name of the file to generate to for this script
DOCSTRING = ""

# The classes and/or the basename of the Doxygen XML files to be processed by
# this script.
ITEMS  = [ 'wxGridCellCoords',

           'wxGridBlockCoords',
           'wxGridBlockDiffResult',
           'wxGridBlocks',

           'wxGridCellRenderer',
           'wxGridCellStringRenderer',
           'wxGridCellAutoWrapStringRenderer',
           'wxGridCellBoolRenderer',
           'wxGridCellDateRenderer',
           'wxGridCellDateTimeRenderer',
           'wxGridCellEnumRenderer',
           'wxGridCellFloatRenderer',
           'wxGridCellNumberRenderer',

           'wxGridActivationResult',
           'wxGridActivationSource',

           'wxGridCellEditor',
           'wxGridCellActivatableEditor',
           'wxGridCellTextEditor',
           'wxGridCellDateEditor',
           'wxGridCellAutoWrapStringEditor',
           'wxGridCellBoolEditor',
           'wxGridCellChoiceEditor',
           'wxGridCellEnumEditor',
           'wxGridCellFloatEditor',
           'wxGridCellNumberEditor',

           'wxGridFitMode',
           'wxGridCellAttr',

           'wxGridCornerHeaderRenderer',
           'wxGridHeaderLabelsRenderer',
           'wxGridRowHeaderRenderer',
           'wxGridColumnHeaderRenderer',
           'wxGridRowHeaderRendererDefault',
           'wxGridColumnHeaderRendererDefault',
           'wxGridCornerHeaderRendererDefault',

           'wxGridCellAttrProvider',
           'wxGridTableBase',
           'wxGridTableMessage',
           'wxGridStringTable',
           'wxGridSizesInfo',

           'wxGrid',
           'wxGridUpdateLocker',

           'wxGridEvent',
           'wxGridSizeEvent',
           'wxGridRangeSelectEvent',
           'wxGridEditorCreatedEvent',

           ]

#---------------------------------------------------------------------------

def run():
    # Parse the XML file(s) building a collection of Extractor objects
    module = etgtools.ModuleDef(PACKAGE, MODULE, NAME, DOCSTRING)
    etgtools.parseDoxyXML(module, ITEMS)

    #-----------------------------------------------------------------
    # Tweak the parsed meta objects in the module object as needed for
    # customizing the generated code and docstrings.

    module.addGlobalStr('wxGridNameStr', module.items[0])
    module.addPyCode("""\
        GRID_VALUE_STRING =    "string"
        GRID_VALUE_BOOL =      "bool"
        GRID_VALUE_NUMBER =    "long"
        GRID_VALUE_FLOAT =     "double"
        GRID_VALUE_CHOICE =    "choice"
        GRID_VALUE_DATE =      "date"
        GRID_VALUE_TEXT =      "string"
        GRID_VALUE_LONG =      "long"
        GRID_VALUE_CHOICEINT = "choiceint"
        GRID_VALUE_DATETIME =  "datetime"
        """)

    # Add compatibility constants for these since they've been removed from wxWidgets
    # TODO: Remove these in a future release.
    module.addPyCode("""\
        GRIDTABLE_REQUEST_VIEW_GET_VALUES = 2000
        GRIDTABLE_REQUEST_VIEW_SEND_VALUES = 2001
        """)

    module.insertItem(0, etgtools.TypedefDef(type='wxWindow', name='wxGridWindow'))

    #-----------------------------------------------------------------
    c = module.find('wxGridCellCoords')
    assert isinstance(c, etgtools.ClassDef)
    tools.addAutoProperties(c)
    c.find('operator!').ignore()
    c.find('operator=').ignore()

    # Add a typemap for 2 element sequences
    c.convertFromPyObject = tools.convertTwoIntegersTemplate('wxGridCellCoords')

    c.addCppMethod('PyObject*', 'Get', '()', """\
        wxPyThreadBlocker blocker;
        return sipBuildResult(0, "(ii)", self->GetRow(), self->GetCol());
        """,
        pyArgsString="() -> (row,col)",
        briefDoc="Return the row and col properties as a tuple.")

    tools.addGetIMMethodTemplate(module, c, ['Row', 'Col'])

    # Add sequence protocol methods and other goodies
    c.addPyMethod('__str__', '(self)',             'return str(self.Get())')
    c.addPyMethod('__repr__', '(self)',            'return "GridCellCoords"+str(self.Get())')
    c.addPyMethod('__len__', '(self)',             'return len(self.Get())')
    c.addPyMethod('__nonzero__', '(self)',         'return self.Get() != (0,0)')
    c.addPyMethod('__bool__', '(self)',            'return self.Get() != (0,0)')
    c.addPyMethod('__reduce__', '(self)',          'return (GridCellCoords, self.Get())')
    c.addPyMethod('__getitem__', '(self, idx)',    'return self.Get()[idx]')
    c.addPyMethod('__setitem__', '(self, idx, val)',
                  """\
                  if idx == 0: self.Row = val
                  elif idx == 1: self.Col = val
                  else: raise IndexError
                  """)
    c.addPyCode('GridCellCoords.__safe_for_unpickling__ = True')

    module.addItem(
        tools.wxArrayWrapperTemplate('wxGridCellCoordsArray', 'wxGridCellCoords', module,
                                     getItemCopy=True))


    #-----------------------------------------------------------------
    c = module.find('wxGridBlockCoords')
    tools.addAutoProperties(c)

    c.find('operator!').ignore()
    c.addCppMethod('int', '__bool__', '()', "return self->operator!();")

    c.addCppMethod('PyObject*', 'Get', '()', """\
        wxPyThreadBlocker blocker;
        return sipBuildResult(0, "(iiii)", self->GetTopRow(), self->GetLeftCol(), self->GetBottomRow(), self->GetRightCol());
        """,
        pyArgsString="() -> (topRow, leftCol, bottomRow, rightCol)",
        briefDoc="Return the block coordinants as a tuple.")

    c.addPyMethod('__str__', '(self)',             'return str(self.Get())')
    c.addPyMethod('__repr__', '(self)',            'return "GridBlockCoords"+str(self.Get())')

    #-----------------------------------------------------------------
    c = module.find('wxGridBlockDiffResult')
    c.find('m_parts').ignore()

    c.addCppMethod('PyObject*', '_getParts', '()',
        """\
        wxPyThreadBlocker blocker;
        PyObject* ret = PyTuple_New(4);
        if (ret) {
            PyTuple_SET_ITEM(ret, 0, wxPyConstructObject(&self->m_parts[0], "wxGridBlockCoords", false));
            PyTuple_SET_ITEM(ret, 1, wxPyConstructObject(&self->m_parts[1], "wxGridBlockCoords", false));
            PyTuple_SET_ITEM(ret, 2, wxPyConstructObject(&self->m_parts[2], "wxGridBlockCoords", false));
            PyTuple_SET_ITEM(ret, 3, wxPyConstructObject(&self->m_parts[3], "wxGridBlockCoords", false));
        }
        return ret;
        """)
    c.addProperty('m_parts', '_getParts')


    #-----------------------------------------------------------------
    c = module.find('wxGridBlocks')
    c.addPrivateDefaultCtor()
    c.addPrivateAssignOp()

    c.addPyMethod('__iter__', '(self)',
                  'return PyGridBlocksIterator(self)',
                  "Returns a Python iterator for accessing the collection of grid blocks.")

    # This class is the Python iterator that knows how to fetch blocks from the
    # wxGridBlocks object
    c.addPyCode("""\
        class PyGridBlocksIterator(object):
            "A Python iterator for GridBlocks objects"
            def __init__(self, blocks):
                self._blocks = blocks
                self._iterator = self._blocks.begin()

            def __next__(self):
                if self._iterator == self._blocks.end():
                    raise StopIteration
                obj = self._iterator._get()
                self._iterator = self._iterator._next()
                return obj
        """)


    # c.find('iterator').addCppMethod('wxGridBlocks::iterator', '_next', '()',
    #                                 "return self->operator++();")
    # c.find('iterator').addCppMethod('const wxGridBlockCoords&', '_get', '()',
    #                                 "return self->operator*();")

    # TODO: Doing these the normal way (above) breaks because it tries to use just
    # "iterator" for the self param type, instead of wxGridBlocks::iterator.
    # That should be fixable, but until then just add these methods the manual
    # sip way.
    c.find('iterator').addItem(etgtools.WigCode("""\
        iterator& _next();
        %MethodCode
            PyErr_Clear();
            Py_BEGIN_ALLOW_THREADS
            sipRes = &sipCpp->operator++();
            Py_END_ALLOW_THREADS
            if (PyErr_Occurred()) return 0;
        %End

        const wxGridBlockCoords& _get() const;
        %MethodCode
            PyErr_Clear();
            Py_BEGIN_ALLOW_THREADS
            sipRes =  new ::wxGridBlockCoords(sipCpp->operator*());
            Py_END_ALLOW_THREADS
            if (PyErr_Occurred()) return 0;
        %End

        bool __eq__(const iterator& it) const;
        bool __ne__(const iterator& it) const;
        """))


    #-----------------------------------------------------------------
    c = module.find('wxGridSizesInfo')
    c.find('m_customSizes').ignore()   # TODO: Add support for wxUnsignedToIntHashMap??


    #-----------------------------------------------------------------
    def fixRendererClass(name):
        klass = module.find(name)
        assert isinstance(klass, etgtools.ClassDef)
        tools.fixRefCountedClass(klass)
        tools.addAutoProperties(klass)

        methods = [
            ('Clone',       "virtual wxGridCellRenderer* Clone() const /Factory/;"),
            ('Draw',        "virtual void Draw(wxGrid& grid, wxGridCellAttr& attr, wxDC& dc, "
                            "    const wxRect& rect, int row, int col, bool isSelected);"),
            ('GetBestSize', "virtual wxSize GetBestSize(wxGrid& grid, wxGridCellAttr& attr, "
                            "    wxDC& dc, int row, int col);"),
            ]
        for method, code in methods:
            if not klass.findItem(method):
                klass.addItem(etgtools.WigCode(code))


    c = module.find('wxGridCellRenderer')
    c.addPrivateCopyCtor()
    c.find('~wxGridCellRenderer').ignore(False)
    c.find('Clone').factory = True
    c.find('Draw').isPureVirtual = False
    tools.fixRefCountedClass(c)

    for name in ITEMS:
        if 'Cell' in name and 'Renderer' in name:
            fixRendererClass(name)

    module.addPyCode("PyGridCellRenderer = wx.deprecated(GridCellRenderer, 'Use GridCellRenderer instead.')")


    #-----------------------------------------------------------------

    c = module.find('wxGridActivationResult')
    c.addPrivateAssignOp()
    c.addPrivateDefaultCtor()
    c.instanceCode = """\
        wxGridActivationResult result = wxGridActivationResult::DoNothing();
        sipCpp = &result;
        """


    c = module.find('wxGridActivationSource')
    c.noDefCtor = True
    c.addPrivateAssignOp()


    #-----------------------------------------------------------------
    def fixEditorClass(name):
        klass = module.find(name)
        assert isinstance(klass, etgtools.ClassDef)
        tools.fixRefCountedClass(klass)
        tools.addAutoProperties(klass)

        methods = [
            ('BeginEdit',  "virtual void BeginEdit(int row, int col, wxGrid* grid);"),
            ('Clone',      "virtual wxGridCellEditor* Clone() const /Factory/;"),
            ('Create',     "virtual void Create(wxWindow* parent, wxWindowID id, wxEvtHandler* evtHandler);"),
            #('EndEdit',    "virtual bool EndEdit(int row, int col, const wxGrid* grid, const wxString& oldval, wxString* newval);"),
            ('ApplyEdit',  "virtual void ApplyEdit(int row, int col, wxGrid* grid);"),
            ('Reset',      "virtual void Reset();"),
            ('GetValue',   "virtual wxString GetValue() const;"),
            ]
        for method, code in methods:
            if not klass.findItem(method):
                klass.addItem(etgtools.WigCode(code))


        # Fix up EndEdit so it returns newval on success or None on failure
        pureVirtual = False
        if klass.findItem('EndEdit'):
            klass.find('EndEdit').ignore()
            pureVirtual = True

        # The Python version of EndEdit has a different signature than the
        # C++ version, so we need to take care of mapping between them so the
        # C++ compiler still recognizes this as a match for the virtual
        # method in the base class.
        klass.addCppMethod('PyObject*', 'EndEdit', '(int row, int col, const wxGrid* grid, const wxString& oldval)',
            cppSignature='bool (int row, int col, const wxGrid* grid, const wxString& oldval, wxString* newval)',
            pyArgsString='(row, col, grid, oldval)',
            isVirtual=True,
            isPureVirtual=pureVirtual,
            doc="""\
                End editing the cell.

                This function must check if the current value of the editing cell
                is valid and different from the original value in its string
                form. If not then simply return None.  If it has changed then
                this method should save the new value so that ApplyEdit can
                apply it later and the string representation of the new value
                should be returned.

                Notice that this method shoiuld not modify the grid as the
                change could still be vetoed.
                """,

            # Code for Python --> C++ calls.  Make it return newval or None.
            body="""\
                bool rv;
                wxString newval;
                rv = self->EndEdit(row, col, grid, *oldval, &newval);
                if (rv) {
                    return wx2PyString(newval);
                }
                else {
                    Py_INCREF(Py_None);
                    return Py_None;
                }
                """,

            # Code for C++ --> Python calls. This is used when a C++ method
            # call needs to be reflected to a call to the overridden Python
            # method, so we need to translate between the real C++ signature
            # and the Python signature.
            virtualCatcherCode="""\
                // VirtualCatcherCode for wx.grid.GridCellEditor.EndEdit
                PyObject *result;
                result = sipCallMethod(0, sipMethod, "iiDN", row, col,
                                       const_cast<wxGrid *>(grid),sipType_wxGrid,NULL,
                                       new wxString(oldval),sipType_wxString,NULL);
                if (result == NULL) {
                    if (PyErr_Occurred())
                        PyErr_Print();
                    sipRes = false;
                }
                else if (result == Py_None) {
                    sipRes = false;
                }
                else {
                    sipRes = true;
                    *newval = Py2wxString(result);
                }
                Py_XDECREF(result);
                """  if pureVirtual else "",  # only used with the base class
            )


    c = module.find('wxGridCellEditor')
    c.addPrivateCopyCtor()
    c.find('~wxGridCellEditor').ignore(False)
    c.find('Clone').factory = True
    tools.fixRefCountedClass(c)


    c = module.find('wxGridCellActivatableEditor')


    c = module.find('wxGridCellChoiceEditor')
    c.find('wxGridCellChoiceEditor').findOverload('count').ignore()

    for name in ITEMS:
        if 'Cell' in name and 'Editor' in name and name != 'wxGridCellActivatableEditor':
            fixEditorClass(name)

    module.addPyCode("PyGridCellEditor = wx.deprecated(GridCellEditor, 'Use GridCellEditor instead.')")

    #-----------------------------------------------------------------
    c = module.find('wxGridCellAttr')
    c.addPrivateCopyCtor()
    c.find('~wxGridCellAttr').ignore(False)
    c.find('Clone').factory = True
    tools.fixRefCountedClass(c)

    c.find('GetAlignment.hAlign').out = True
    c.find('GetAlignment.vAlign').out = True
    c.find('GetNonDefaultAlignment.hAlign').out = True
    c.find('GetNonDefaultAlignment.vAlign').out = True
    c.find('GetSize.num_rows').out = True
    c.find('GetSize.num_cols').out = True

    c.find('SetEditor.editor').transfer = True  # these are probably redundant now...
    c.find('SetRenderer.renderer').transfer = True

    c.find('GetRendererPtr').ignore()

    #-----------------------------------------------------------------
    module.find('wxGridCellRendererPtr').piIgnored = True
    module.find('wxGridCellEditorPtr').piIgnored = True
    module.find('wxGridCellAttrPtr').piIgnored = True

    #-----------------------------------------------------------------
    # The instanceCode attribute is code that is used to make a default
    # instance of the class. We can't create them using the same class in
    # this case because they are abstract.

    c = module.find('wxGridCornerHeaderRenderer')
    c.instanceCode = 'sipCpp = new wxGridCornerHeaderRendererDefault;'

    c = module.find('wxGridRowHeaderRenderer')
    c.instanceCode = 'sipCpp = new wxGridRowHeaderRendererDefault;'

    c = module.find('wxGridColumnHeaderRenderer')
    c.instanceCode = 'sipCpp = new wxGridColumnHeaderRendererDefault;'



    #-----------------------------------------------------------------
    c = module.find('wxGridCellAttrProvider')
    c.addPrivateCopyCtor()

    c.find('SetAttr.attr').transfer = True
    c.find('SetRowAttr.attr').transfer = True
    c.find('SetColAttr.attr').transfer = True

    module.addPyCode("PyGridCellAttrProvider = wx.deprecated(GridCellAttrProvider, 'Use GridCellAttrProvider instead.')")


    #-----------------------------------------------------------------
    c = module.find('wxGridTableBase')
    c.addPrivateCopyCtor()

    c.find('SetAttr.attr').transfer = True
    c.find('SetRowAttr.attr').transfer = True
    c.find('SetColAttr.attr').transfer = True

    module.addPyCode("PyGridTableBase = wx.deprecated(GridTableBase, 'Use GridTableBase instead.')")


    # Make the GetValue methods easier to use from Python.  For example,
    # instead of needing to always return a string, the GetValue in the derived
    # class can return any type (as long as the renderer and editor knows how
    # to deal with it, and the value can be converted to a string for display).
    m = c.find('GetValue')
    m.type = 'PyObject*'
    m.cppSignature = 'wxString (int row, int col)'
    m.setCppCode("return wx2PyString(self->GetValue(row, col));")
    m.virtualCatcherCode = """\
        // virtualCatcherCode for GridTableBase.GetValue
        PyObject *result = sipCallMethod(&sipIsErr, sipMethod, "ii", row, col);
        if (result == NULL) {
            if (PyErr_Occurred())
                PyErr_Print();
            sipRes = "";
        }
        else if (result == Py_None) {
            sipRes = "";
        }
        else {
            if (!PyBytes_Check(result) && !PyUnicode_Check(result)) {
                PyObject* old = result;
                result = PyObject_Str(result);
                Py_DECREF(old);
            }
            sipRes = Py2wxString(result);
        }
        Py_XDECREF(result);
        """

    # SetValue is okay as-is...


    # Replace these virtuals in the base class with Python methods, they just
    # need to call GetValue or SetValue directly since they must already be
    # implemented in the derived Python class because they are pure virtual.
    c.addPyMethod('GetValueAsLong', '(self, row, col)',
        body="""\
            val = self.GetValue(row, col)
            try:
                return int(val)
            except ValueError:
                return 0
            """, docsIgnored=True)

    c.addPyMethod('GetValueAsDouble', '(self, row, col)',
        body="""\
            val = self.GetValue(row, col)
            try:
                return float(val)
            except ValueError:
                return 0.0
            """, docsIgnored=True)

    c.addPyMethod('GetValueAsBool', '(self, row, col)',
        body="""\
            val = self.GetValue(row, col)
            try:
                return bool(val)
            except ValueError:
                return False
            """, docsIgnored=True)

    c.addPyMethod('SetValueAsLong', '(self, row, col, value)',
        body="self.SetValue(row, col, int(value))", docsIgnored=True)

    c.addPyMethod('SetValueAsDouble', '(self, row, col, value)',
        body="self.SetValue(row, col, float(value))", docsIgnored=True)

    c.addPyMethod('SetValueAsBool', '(self, row, col, value)',
        body="self.SetValue(row, col, bool(value))", docsIgnored=True)


    # Should we add support for using generic PyObjects in the *AsCustom
    # methods? I don't think it is necessary due to the GetValue
    # modifications above, so for now, at least, let.s just ignore them.
    c.find('GetValueAsCustom').ignore()
    c.find('SetValueAsCustom').ignore()


    #-----------------------------------------------------------------
    c = module.find('wxGridStringTable')
    c.addPrivateCopyCtor()


    #-----------------------------------------------------------------
    c = module.find('wxGridTableMessage')
    c.addPrivateCopyCtor()


    #-----------------------------------------------------------------
    c = module.find('wxGrid')
    tools.fixWindowClass(c, ignoreProtected=False)
    c.bases = ['wxScrolledCanvas']

    c.find('GetColLabelAlignment.horiz').out = True
    c.find('GetColLabelAlignment.vert').out = True
    c.find('GetRowLabelAlignment.horiz').out = True
    c.find('GetRowLabelAlignment.vert').out = True

    c.find('GetCellAlignment.horiz').out = True
    c.find('GetCellAlignment.vert').out = True
    c.find('GetDefaultCellAlignment.horiz').out = True
    c.find('GetDefaultCellAlignment.vert').out = True


    c.find('RegisterDataType.renderer').transfer = True
    c.find('RegisterDataType.editor').transfer = True

    c.find('SetRowAttr.attr').transfer = True
    c.find('SetColAttr.attr').transfer = True
    c.find('SetCellEditor.editor').transfer = True
    c.find('SetCellRenderer.renderer').transfer = True

    # This overload is deprecated, so don't generate code for it.
    c.find('SetCellValue').findOverload('wxString &val').ignore()

    c.find('SetDefaultEditor.editor').transfer = True
    c.find('SetDefaultRenderer.renderer').transfer = True

    for n in ['GetColGridLinePen', 'GetDefaultGridLinePen', 'GetRowGridLinePen']:
        c.find(n).isVirtual = True


    # The SetTable method can optionally pass ownership of the table
    # object to the grid, so we need to optionally update the
    # ownership of the Python proxy object to match.
    c.find('SetTable').pyName = '_SetTable'
    c.addPyMethod('SetTable', '(self, table, takeOwnership=False, selmode=Grid.GridSelectCells)',
        piArgsString='(self, table, takeOwnership=False, selmode=GridSelectCells)',
        doc="Set the Grid Table to be used by this grid.",
        body="""\
            val = self._SetTable(table, takeOwnership, selmode)
            if takeOwnership:
                import wx.siplib
                wx.siplib.transferto(table, self)
            return val
        """)


    # SIP will normally try to add support for overriding this method since
    # it is inherited from super classes, but in this case we want it to be
    # ignored (because IRL it is private in one of the intermediate classes)
    # so we'll tell SIP that it is private here instead.
    c.addItem(etgtools.WigCode("""\
        wxSize GetSizeAvailableForScrollTarget(const wxSize& size);
        """, protection='private'))


    # Add a simpler set of names for the wxGridSelectionModes enum
    c.addPyCode("""\
        Grid.SelectCells = Grid.GridSelectCells
        Grid.SelectRows = Grid.GridSelectRows
        Grid.SelectColumns = Grid.GridSelectColumns
        Grid.SelectRowsOrColumns = Grid.GridSelectRowsOrColumns
        """)


    c.find('SetCellAlignment').findOverload('align').ignore()
    c.find('SetCellTextColour').overloads = []

    c.find('GetGridWindowOffset').findOverload('int &x').ignore()


    # Custom code to deal with the wxGridBlockCoordsVector return type of these
    # methods. It's a wxVector, which we'll just convert to a list.

    # TODO: There are a few of these now so we ought to either wrap wxVector, or add
    #       something in tweaker_tools to make adding code like this easier and more
    #       automated.
    code = """\
        wxPyThreadBlocker blocker;
        PyObject* result = PyList_New(0);
        wxGridBlockCoordsVector vector = self->{method}();
        for (size_t idx=0; idx < vector.size(); idx++) {{
            PyObject* obj;
            wxGridBlockCoords* item = new wxGridBlockCoords(vector[idx]);
            obj = wxPyConstructObject((void*)item, "wxGridBlockCoords", true);
            PyList_Append(result, obj);
            Py_DECREF(obj);
        }}
        return result;
        """
    c.find('GetSelectedRowBlocks').type = 'PyObject*'
    c.find('GetSelectedRowBlocks').setCppCode(code.format(method='GetSelectedRowBlocks'))
    c.find('GetSelectedColBlocks').type = 'PyObject*'
    c.find('GetSelectedColBlocks').setCppCode(code.format(method='GetSelectedColBlocks'))


    #-----------------------------------------------------------------
    c = module.find('wxGridUpdateLocker')
    c.addPrivateCopyCtor()

    # context manager methods
    c.addPyMethod('__enter__', '(self)', 'return self')
    c.addPyMethod('__exit__', '(self, exc_type, exc_val, exc_tb)', 'return False')


    #-----------------------------------------------------------------

    for name in ['wxGridSizeEvent',
                 'wxGridRangeSelectEvent',
                 'wxGridEditorCreatedEvent',
                 'wxGridEvent'
                 ]:
        c = module.find(name)
        tools.fixEventClass(c)


    module.addPyCode("""\
        EVT_GRID_CELL_LEFT_CLICK = wx.PyEventBinder( wxEVT_GRID_CELL_LEFT_CLICK )
        EVT_GRID_CELL_RIGHT_CLICK = wx.PyEventBinder( wxEVT_GRID_CELL_RIGHT_CLICK )
        EVT_GRID_CELL_LEFT_DCLICK = wx.PyEventBinder( wxEVT_GRID_CELL_LEFT_DCLICK )
        EVT_GRID_CELL_RIGHT_DCLICK = wx.PyEventBinder( wxEVT_GRID_CELL_RIGHT_DCLICK )
        EVT_GRID_LABEL_LEFT_CLICK = wx.PyEventBinder( wxEVT_GRID_LABEL_LEFT_CLICK )
        EVT_GRID_LABEL_RIGHT_CLICK = wx.PyEventBinder( wxEVT_GRID_LABEL_RIGHT_CLICK )
        EVT_GRID_LABEL_LEFT_DCLICK = wx.PyEventBinder( wxEVT_GRID_LABEL_LEFT_DCLICK )
        EVT_GRID_LABEL_RIGHT_DCLICK = wx.PyEventBinder( wxEVT_GRID_LABEL_RIGHT_DCLICK )
        EVT_GRID_ROW_SIZE = wx.PyEventBinder( wxEVT_GRID_ROW_SIZE )
        EVT_GRID_COL_SIZE = wx.PyEventBinder( wxEVT_GRID_COL_SIZE )
        EVT_GRID_COL_AUTO_SIZE = wx.PyEventBinder( wxEVT_GRID_COL_AUTO_SIZE )
        EVT_GRID_RANGE_SELECTING = wx.PyEventBinder( wxEVT_GRID_RANGE_SELECTING )
        EVT_GRID_RANGE_SELECTED = wx.PyEventBinder( wxEVT_GRID_RANGE_SELECTED )
        EVT_GRID_CELL_CHANGING = wx.PyEventBinder( wxEVT_GRID_CELL_CHANGING )
        EVT_GRID_CELL_CHANGED = wx.PyEventBinder( wxEVT_GRID_CELL_CHANGED )
        EVT_GRID_SELECT_CELL = wx.PyEventBinder( wxEVT_GRID_SELECT_CELL )
        EVT_GRID_EDITOR_SHOWN = wx.PyEventBinder( wxEVT_GRID_EDITOR_SHOWN )
        EVT_GRID_EDITOR_HIDDEN = wx.PyEventBinder( wxEVT_GRID_EDITOR_HIDDEN )
        EVT_GRID_EDITOR_CREATED = wx.PyEventBinder( wxEVT_GRID_EDITOR_CREATED )
        EVT_GRID_CELL_BEGIN_DRAG = wx.PyEventBinder( wxEVT_GRID_CELL_BEGIN_DRAG )
        EVT_GRID_ROW_MOVE = wx.PyEventBinder( wxEVT_GRID_ROW_MOVE )
        EVT_GRID_COL_MOVE = wx.PyEventBinder( wxEVT_GRID_COL_MOVE )
        EVT_GRID_COL_SORT = wx.PyEventBinder( wxEVT_GRID_COL_SORT )
        EVT_GRID_TABBING = wx.PyEventBinder( wxEVT_GRID_TABBING )

        # The same as above but with the ability to specify an identifier
        EVT_GRID_CMD_CELL_LEFT_CLICK =     wx.PyEventBinder( wxEVT_GRID_CELL_LEFT_CLICK,    1 )
        EVT_GRID_CMD_CELL_RIGHT_CLICK =    wx.PyEventBinder( wxEVT_GRID_CELL_RIGHT_CLICK,   1 )
        EVT_GRID_CMD_CELL_LEFT_DCLICK =    wx.PyEventBinder( wxEVT_GRID_CELL_LEFT_DCLICK,   1 )
        EVT_GRID_CMD_CELL_RIGHT_DCLICK =   wx.PyEventBinder( wxEVT_GRID_CELL_RIGHT_DCLICK,  1 )
        EVT_GRID_CMD_LABEL_LEFT_CLICK =    wx.PyEventBinder( wxEVT_GRID_LABEL_LEFT_CLICK,   1 )
        EVT_GRID_CMD_LABEL_RIGHT_CLICK =   wx.PyEventBinder( wxEVT_GRID_LABEL_RIGHT_CLICK,  1 )
        EVT_GRID_CMD_LABEL_LEFT_DCLICK =   wx.PyEventBinder( wxEVT_GRID_LABEL_LEFT_DCLICK,  1 )
        EVT_GRID_CMD_LABEL_RIGHT_DCLICK =  wx.PyEventBinder( wxEVT_GRID_LABEL_RIGHT_DCLICK, 1 )
        EVT_GRID_CMD_ROW_SIZE =            wx.PyEventBinder( wxEVT_GRID_ROW_SIZE,           1 )
        EVT_GRID_CMD_COL_SIZE =            wx.PyEventBinder( wxEVT_GRID_COL_SIZE,           1 )
        EVT_GRID_CMD_COL_AUTO_SIZE =       wx.PyEventBinder( wxEVT_GRID_COL_AUTO_SIZE,      1 )
        EVT_GRID_CMD_RANGE_SELECTING =     wx.PyEventBinder( wxEVT_GRID_RANGE_SELECTING,    1 )
        EVT_GRID_CMD_RANGE_SELECTED =      wx.PyEventBinder( wxEVT_GRID_RANGE_SELECTED,     1 )
        EVT_GRID_CMD_CELL_CHANGING =       wx.PyEventBinder( wxEVT_GRID_CELL_CHANGING,      1 )
        EVT_GRID_CMD_CELL_CHANGED =        wx.PyEventBinder( wxEVT_GRID_CELL_CHANGED,       1 )
        EVT_GRID_CMD_SELECT_CELL =         wx.PyEventBinder( wxEVT_GRID_SELECT_CELL,        1 )
        EVT_GRID_CMD_EDITOR_SHOWN =        wx.PyEventBinder( wxEVT_GRID_EDITOR_SHOWN,       1 )
        EVT_GRID_CMD_EDITOR_HIDDEN =       wx.PyEventBinder( wxEVT_GRID_EDITOR_HIDDEN,      1 )
        EVT_GRID_CMD_EDITOR_CREATED =      wx.PyEventBinder( wxEVT_GRID_EDITOR_CREATED,     1 )
        EVT_GRID_CMD_CELL_BEGIN_DRAG =     wx.PyEventBinder( wxEVT_GRID_CELL_BEGIN_DRAG,    1 )
        EVT_GRID_CMD_ROW_MOVE =            wx.PyEventBinder( wxEVT_GRID_ROW_MOVE,           1 )
        EVT_GRID_CMD_COL_MOVE =            wx.PyEventBinder( wxEVT_GRID_COL_MOVE,           1 )
        EVT_GRID_CMD_COL_SORT =            wx.PyEventBinder( wxEVT_GRID_COL_SORT,           1 )
        EVT_GRID_CMD_TABBING =             wx.PyEventBinder( wxEVT_GRID_TABBING,            1 )

        # Just for compatibility, remove them in a future release
        EVT_GRID_RANGE_SELECT =            EVT_GRID_RANGE_SELECTED
        EVT_GRID_CMD_RANGE_SELECT =        EVT_GRID_CMD_RANGE_SELECTED
        wxEVT_GRID_RANGE_SELECT =          wxEVT_GRID_RANGE_SELECTED

        """)

    #-----------------------------------------------------------------
    #-----------------------------------------------------------------
    tools.doCommonTweaks(module)
    tools.runGenerators(module)


#---------------------------------------------------------------------------
if __name__ == '__main__':
    run()




#---------------------------------------------------------------------------
# Name:        etg/_richtext.py
# Author:      Robin Dunn
#
# Created:     27-Oct-2012
# Copyright:   (c) 2012-2020 by Total Control Software
# License:     wxWindows License
#---------------------------------------------------------------------------

import etgtools
import etgtools.tweaker_tools as tools

PACKAGE   = "wx"
MODULE    = "_richtext"
NAME      = "_richtext"   # Base name of the file to generate to for this script
DOCSTRING = """\
The :class:`RichTextCtrl` is a generic, ground-up implementation of a rich
text control capable of showing multiple text styles and images.  This module
contains the control and many supporting classes needed for using the features
of the :class:`RichTextCtrl`.

.. note:: Due to some internal dynamic initialization in wxWidgets, this
          module should be imported **before** the :class:`wx.App` object is
          created.
"""

# The classes and/or the basename of the Doxygen XML files to be processed by
# this script.
ITEMS  = [ ]


# The list of other ETG scripts and back-end generator modules that are
# included as part of this module. These should all be items that are put in
# the wxWidgets "richtext" library in a multi-lib build.
INCLUDES = [ 'richtextbuffer',
             'richtextctrl',
             'richtexthtml',
             'richtextxml',
             'richtextprint',
             'richtextstyles',
             'richtextstyledlg',
             'richtextsymboldlg',
             'richtextformatdlg',
             ]


# Separate the list into those that are generated from ETG scripts and the
# rest. These lists can be used from the build scripts to get a list of
# sources and/or additional dependencies when building this extension module.
ETGFILES = ['etg/%s.py' % NAME] + tools.getEtgFiles(INCLUDES)
DEPENDS = tools.getNonEtgFiles(INCLUDES)
OTHERDEPS = [  ]


#---------------------------------------------------------------------------

def run():
    # Parse the XML file(s) building a collection of Extractor objects
    module = etgtools.ModuleDef(PACKAGE, MODULE, NAME, DOCSTRING,
                                check4unittest = False)
    etgtools.parseDoxyXML(module, ITEMS)

    #-----------------------------------------------------------------
    # Tweak the parsed meta objects in the module object as needed for
    # customizing the generated code and docstrings.

    module.addHeaderCode('#include <wxPython/wxpy_api.h>')
    module.addImport('_core')
    module.addPyCode("import wx", order=10)
    module.addImport('_xml')
    module.addImport('_html')
    module.addImport('_adv')

    module.addInclude(INCLUDES)

    # Redo the initialization of wxModules in the case where this extension
    # module is not imported until *after* the wx.App has been created.
    module.addInitializerCode("""\
        wxPyReinitializeModules();
        """)


    #-----------------------------------------------------------------
    tools.doCommonTweaks(module)
    tools.runGenerators(module)



#---------------------------------------------------------------------------

if __name__ == '__main__':
    run()



#!/usr/bin/env python

import wx

#---------------------------------------------------------------------------

class TestPanel(wx.Panel):
    def __init__(self, parent, log):
        self.log = log
        wx.Panel.__init__(self, parent, -1)

        b = wx.Button(self, -1, "Create and Show a MessageDialog", (50,50))
        self.Bind(wx.EVT_BUTTON, self.OnButton, b)


    def OnButton(self, evt):
        dlg = wx.MessageDialog(self, 'Hello from Python and wxPython!',
                               'A Message Box',
                               wx.OK | wx.ICON_INFORMATION
                               #wx.YES_NO | wx.NO_DEFAULT | wx.CANCEL | wx.ICON_INFORMATION
                               )
        dlg.ShowModal()
        dlg.Destroy()


#---------------------------------------------------------------------------


def runTest(frame, nb, log):
    win = TestPanel(nb, log)
    return win


#---------------------------------------------------------------------------



overview = """\
<html><body>
<h2>wx.MessageDialog</h2>

This class represents a dialog that shows a single or multi-line
message, with a choice of OK, Yes, No and Cancel buttons.
Additionally, various style flags can determine whether an icon is
displayed, and, if so, what kind.  The return value of ShowModal
indicates which button was pressed.

"""


if __name__ == '__main__':
    import sys,os
    import run
    run.main(['', os.path.basename(sys.argv[0])] + sys.argv[1:])



import unittest
from unittests import wtc
import wx
import wx.lib.expando

#---------------------------------------------------------------------------

class lib_expando_Tests(wtc.WidgetTestCase):

    def test_lib_expando1(self):
        pnl = wx.Panel(self.frame)
        w = wx.lib.expando.ExpandoTextCtrl(pnl, value="This is a test", pos=(10,10))
        bs1 = w.GetSize()

        w.AppendText("\nThis is a New Label")
        bs2 = w.GetSize()

        self.assertEqual(w.GetValue(), "This is a test\nThis is a New Label")

        # All we can test here is that we have more lines than we started
        # with, since different platforms may wrap at different spots in the
        # string.
        self.assertGreaterEqual(w.GetNumberOfLines(), 2)
        self.assertGreater(bs2.height, bs1.height)

#---------------------------------------------------------------------------

if __name__ == '__main__':
    unittest.main()



import unittest
from unittests import wtc
import wx
import os

import wx.lib.agw.aui as aui

#---------------------------------------------------------------------------

class lib_agw_aui_Tests(wtc.WidgetTestCase):

    def test_lib_agw_auiCtor(self):
        self._mgr = aui.AuiManager()

        # tell AuiManager to manage this frame
        self._mgr.SetManagedWindow(self.frame)

        pane = wx.Panel()
        self._mgr.AddPane(pane, aui.AuiPaneInfo().Name("pane1").Caption("A pane")
                          .CenterPane())
        pane = wx.Panel()
        self._mgr.AddPane(pane, aui.AuiPaneInfo().Name("pane2").Caption("A pane")
                          .Top())
        pane = wx.Panel()
        self._mgr.AddPane(pane, aui.AuiPaneInfo().Name("pane3").Caption("A pane")
                          .Left())
        pane = wx.Panel()
        self._mgr.AddPane(pane, aui.AuiPaneInfo().Name("pane4").Caption("A pane")
                          .Bottom())
        self._mgr.Update()

    def tearDown(self):
        self._mgr.UnInit()


class lib_agw_aui_additional_Tests(wtc.WidgetTestCase):

    def test_lib_agw_aui_ToolbarCtor(self):
        tb = aui.AuiToolBar(self.frame, -1, wx.DefaultPosition, wx.DefaultSize,
                             agwStyle=aui.AUI_TB_OVERFLOW | aui.AUI_TB_VERT_TEXT)
        tb_bmp1 = wx.ArtProvider.GetBitmap(wx.ART_QUESTION, wx.ART_OTHER, wx.Size(16, 16))
        tb.AddSimpleTool(-1, "Test", tb_bmp1)
        tb.AddSimpleTool(-1, "Check 1", tb_bmp1, "Check 1", aui.ITEM_CHECK)
        tb.AddSimpleTool(-1, "Radio 1", tb_bmp1, "Radio 1", aui.ITEM_RADIO)
        tb.AddSeparator()

        # prepare a few custom overflow elements for the toolbars' overflow buttons

        prepend_items, append_items = [], []
        item = aui.AuiToolBarItem()

        item.SetKind(wx.ITEM_SEPARATOR)
        append_items.append(item)

        item = aui.AuiToolBarItem()
        item.SetKind(wx.ITEM_NORMAL)
        item.SetId(-1)
        item.SetLabel("Customize...")
        append_items.append(item)
        tb.SetCustomOverflowItems(prepend_items, append_items)
        tb.Realize()


    def test_lib_agw_auiEvents(self):
        aui.EVT_AUI_PANE_BUTTON
        """ Fires an event when the user left-clicks on a pane button. """
        aui.EVT_AUI_PANE_CLOSE
        """ A pane in `AuiManager` has been closed. """
        aui.EVT_AUI_PANE_MAXIMIZE
        """ A pane in `AuiManager` has been maximized. """
        aui.EVT_AUI_PANE_RESTORE
        """ A pane in `AuiManager` has been restored from a maximized state. """
        aui.EVT_AUI_RENDER
        """ Fires an event every time the AUI frame is being repainted. """
        aui.EVT_AUI_FIND_MANAGER
        """ Used to find which AUI manager is controlling a certain pane. """
        aui.EVT_AUI_PANE_MINIMIZE
        """ A pane in `AuiManager` has been minimized. """
        aui.EVT_AUI_PANE_MIN_RESTORE
        """ A pane in `AuiManager` has been restored from a minimized state. """
        aui.EVT_AUI_PANE_FLOATING
        """ A pane in `AuiManager` is about to be floated. """
        aui.EVT_AUI_PANE_FLOATED
        """ A pane in `AuiManager` has been floated. """
        aui.EVT_AUI_PANE_DOCKING
        """ A pane in `AuiManager` is about to be docked. """
        aui.EVT_AUI_PANE_DOCKED
        """ A pane in `AuiManager` has been docked. """
        aui.EVT_AUI_PANE_ACTIVATED
        """ A pane in `AuiManager` has been activated. """
        aui.EVT_AUI_PERSPECTIVE_CHANGED
        """ The layout in `AuiManager` has been changed. """

    def test_lib_agw_auiConstantsExist(self):

        # For auibook
        # -----------
        aui.AuiBaseTabCtrlId
        """ Base window identifier for AuiTabCtrl. """

        aui.AUI_NB_TOP
        """ With this style, tabs are drawn along the top of the notebook. """
        aui.AUI_NB_LEFT
        """ With this style, tabs are drawn along the left of the notebook.
        Not implemented yet. """
        aui.AUI_NB_RIGHT
        """ With this style, tabs are drawn along the right of the notebook.
        Not implemented yet. """
        aui.AUI_NB_BOTTOM
        """ With this style, tabs are drawn along the bottom of the notebook. """
        aui.AUI_NB_TAB_SPLIT
        """ Allows the tab control to be split by dragging a tab. """
        aui.AUI_NB_TAB_MOVE
        """ Allows a tab to be moved horizontally by dragging. """
        aui.AUI_NB_TAB_EXTERNAL_MOVE
        """ Allows a tab to be moved to another tab control. """
        aui.AUI_NB_TAB_FIXED_WIDTH
        """ With this style, all tabs have the same width. """
        aui.AUI_NB_SCROLL_BUTTONS
        """ With this style, left and right scroll buttons are displayed. """
        aui.AUI_NB_WINDOWLIST_BUTTON
        """ With this style, a drop-down list of windows is available. """
        aui.AUI_NB_CLOSE_BUTTON
        """ With this style, a close button is available on the tab bar. """
        aui.AUI_NB_CLOSE_ON_ACTIVE_TAB
        """ With this style, a close button is available on the active tab. """
        aui.AUI_NB_CLOSE_ON_ALL_TABS
        """ With this style, a close button is available on all tabs. """
        aui.AUI_NB_MIDDLE_CLICK_CLOSE
        """ Allows to close `AuiNotebook` tabs by mouse middle button click. """
        aui.AUI_NB_SUB_NOTEBOOK
        """ This style is used by `AuiManager` to create automatic `AuiNotebooks`. """
        aui.AUI_NB_HIDE_ON_SINGLE_TAB
        """ Hides the tab window if only one tab is present. """
        aui.AUI_NB_SMART_TABS
        """ Use `Smart Tabbing`, like ``Alt`` + ``Tab`` on Windows. """
        aui.AUI_NB_USE_IMAGES_DROPDOWN
        """ Uses images on dropdown window list menu instead of check items. """
        aui.AUI_NB_CLOSE_ON_TAB_LEFT
        """ Draws the tab close button on the left instead of on the right
        (a la Camino browser). """
        aui.AUI_NB_TAB_FLOAT
        """ Allows the floating of single tabs.
        Known limitation: when the notebook is more or less full screen, tabs
        cannot be dragged far enough outside of the notebook to become
        floating pages. """
        aui.AUI_NB_DRAW_DND_TAB
        """ Draws an image representation of a tab while dragging. """
        aui.AUI_NB_ORDER_BY_ACCESS
        """ Tab navigation order by last access time. """
        aui.AUI_NB_NO_TAB_FOCUS
        """ Don't draw tab focus rectangle. """

        aui.AUI_NB_DEFAULT_STYLE
        """ Default `AuiNotebook` style. """

        # -------------------------- #
        # - FrameManager Constants - #
        # -------------------------- #

        # Docking Styles
        aui.AUI_DOCK_NONE
        """ No docking direction. """
        aui.AUI_DOCK_TOP
        """ Top docking direction. """
        aui.AUI_DOCK_RIGHT
        """ Right docking direction. """
        aui.AUI_DOCK_BOTTOM
        """ Bottom docking direction. """
        aui.AUI_DOCK_LEFT
        """ Left docking direction. """
        aui.AUI_DOCK_CENTER
        """ Center docking direction. """
        aui.AUI_DOCK_CENTRE
        """ Centre docking direction. """
        aui.AUI_DOCK_NOTEBOOK_PAGE
        """ Automatic AuiNotebooks docking style. """

        # Floating/Dragging Styles
        aui.AUI_MGR_ALLOW_FLOATING
        """ Allow floating of panes. """
        aui.AUI_MGR_ALLOW_ACTIVE_PANE
        """ If a pane becomes active, "highlight" it in the interface. """
        aui.AUI_MGR_TRANSPARENT_DRAG
        """ If the platform supports it, set transparency on a floating pane
        while it is dragged by the user. """
        aui.AUI_MGR_TRANSPARENT_HINT
        """ If the platform supports it, show a transparent hint window when
        the user is about to dock a floating pane. """
        aui.AUI_MGR_VENETIAN_BLINDS_HINT
        """ Show a "venetian blind" effect when the user is about to dock a
        floating pane. """
        aui.AUI_MGR_RECTANGLE_HINT
        """ Show a rectangle hint effect when the user is about to dock a
        floating pane. """
        aui.AUI_MGR_HINT_FADE
        """ If the platform supports it, the hint window will fade in and out. """
        aui.AUI_MGR_NO_VENETIAN_BLINDS_FADE
        """ Disables the "venetian blind" fade in and out. """
        aui.AUI_MGR_LIVE_RESIZE
        """ Live resize when the user drag a sash. """
        aui.AUI_MGR_ANIMATE_FRAMES
        """ Fade-out floating panes when they are closed (all platforms which support
        frames transparency) and show a moving rectangle when they are docked
        (Windows < Vista and GTK only). """
        aui.AUI_MGR_AERO_DOCKING_GUIDES
        """ Use the new Aero-style bitmaps as docking guides. """
        aui.AUI_MGR_PREVIEW_MINIMIZED_PANES
        """ Slide in and out minimized panes to preview them. """
        aui.AUI_MGR_WHIDBEY_DOCKING_GUIDES
        """ Use the new Whidbey-style bitmaps as docking guides. """
        aui.AUI_MGR_SMOOTH_DOCKING
        """ Performs a "smooth" docking of panes (a la PyQT). """
        aui.AUI_MGR_USE_NATIVE_MINIFRAMES
        """ Use miniframes with native caption bar as floating panes instead or custom
        drawn caption bars (forced on wxMac). """
        aui.AUI_MGR_AUTONB_NO_CAPTION
        """ Panes that merge into an automatic notebook will not have the pane
        caption visible. """


        aui.AUI_MGR_DEFAULT
        """ Default `AuiManager` style. """

        # Panes Customization
        aui.AUI_DOCKART_SASH_SIZE
        """ Customizes the sash size. """
        aui.AUI_DOCKART_CAPTION_SIZE
        """ Customizes the caption size. """
        aui.AUI_DOCKART_GRIPPER_SIZE
        """ Customizes the gripper size. """
        aui.AUI_DOCKART_PANE_BORDER_SIZE
        """ Customizes the pane border size. """
        aui.AUI_DOCKART_PANE_BUTTON_SIZE
        """ Customizes the pane button size. """
        aui.AUI_DOCKART_BACKGROUND_COLOUR
        """ Customizes the background colour. """
        aui.AUI_DOCKART_BACKGROUND_GRADIENT_COLOUR
        """ Customizes the background gradient colour. """
        aui.AUI_DOCKART_SASH_COLOUR
        """ Customizes the sash colour. """
        aui.AUI_DOCKART_ACTIVE_CAPTION_COLOUR
        """ Customizes the active caption colour. """
        aui.AUI_DOCKART_ACTIVE_CAPTION_GRADIENT_COLOUR
        """ Customizes the active caption gradient colour. """
        aui.AUI_DOCKART_INACTIVE_CAPTION_COLOUR
        """ Customizes the inactive caption colour. """
        aui.AUI_DOCKART_INACTIVE_CAPTION_GRADIENT_COLOUR
        """ Customizes the inactive gradient caption colour. """
        aui.AUI_DOCKART_ACTIVE_CAPTION_TEXT_COLOUR
        """ Customizes the active caption text colour. """
        aui.AUI_DOCKART_INACTIVE_CAPTION_TEXT_COLOUR
        """ Customizes the inactive caption text colour. """
        aui.AUI_DOCKART_BORDER_COLOUR
        """ Customizes the border colour. """
        aui.AUI_DOCKART_GRIPPER_COLOUR
        """ Customizes the gripper colour. """
        aui.AUI_DOCKART_CAPTION_FONT
        """ Customizes the caption font. """
        aui.AUI_DOCKART_GRADIENT_TYPE
        """ Customizes the gradient type (no gradient, vertical or horizontal). """
        aui.AUI_DOCKART_DRAW_SASH_GRIP
        """ Draw a sash grip on the sash. """
        aui.AUI_DOCKART_HINT_WINDOW_COLOUR
        """ Customizes the hint window background colour (currently light blue). """

        # Caption Gradient Type
        aui.AUI_GRADIENT_NONE
        """ No gradient on the captions. """
        aui.AUI_GRADIENT_VERTICAL

        """ Vertical gradient on the captions. """
        aui.AUI_GRADIENT_HORIZONTAL
        """ Horizontal gradient on the captions. """

        # Pane Button State
        aui.AUI_BUTTON_STATE_NORMAL
        """ Normal button state. """
        aui.AUI_BUTTON_STATE_HOVER
        """ Hovered button state. """
        aui.AUI_BUTTON_STATE_PRESSED
        """ Pressed button state. """
        aui.AUI_BUTTON_STATE_DISABLED
        """ Disabled button state. """
        aui.AUI_BUTTON_STATE_HIDDEN
        """ Hidden button state. """
        aui.AUI_BUTTON_STATE_CHECKED
        """ Checked button state. """

        # Pane minimize mode
        aui.AUI_MINIMIZE_POS_SMART
        """ Minimizes the pane on the closest tool bar. """
        aui.AUI_MINIMIZE_POS_TOP
        """ Minimizes the pane on the top tool bar. """
        aui.AUI_MINIMIZE_POS_LEFT
        """ Minimizes the pane on its left tool bar. """
        aui.AUI_MINIMIZE_POS_RIGHT
        """ Minimizes the pane on its right tool bar. """
        aui.AUI_MINIMIZE_POS_BOTTOM
        """ Minimizes the pane on its bottom tool bar. """
        aui.AUI_MINIMIZE_POS_TOOLBAR
        """ Minimizes the pane on its bottom tool bar. """
        aui.AUI_MINIMIZE_POS_MASK
        """ Mask to filter the position flags. """
        aui.AUI_MINIMIZE_CAPT_HIDE
        """ Hides the caption of the minimized pane. """
        aui.AUI_MINIMIZE_CAPT_SMART
        """ Displays the caption in the best rotation (horz or clockwise). """
        aui.AUI_MINIMIZE_CAPT_HORZ
        """ Displays the caption horizontally. """
        aui.AUI_MINIMIZE_CAPT_MASK
        """ Mask to filter the caption flags. """

        # Button kind
        aui.AUI_BUTTON_CLOSE
        """ Shows a close button on the pane. """
        aui.AUI_BUTTON_MAXIMIZE_RESTORE
        """ Shows a maximize/restore button on the pane. """
        aui.AUI_BUTTON_MINIMIZE
        """ Shows a minimize button on the pane. """
        aui.AUI_BUTTON_PIN
        """ Shows a pin button on the pane. """
        aui.AUI_BUTTON_OPTIONS
        """ Shows an option button on the pane (not implemented). """
        aui.AUI_BUTTON_WINDOWLIST
        """ Shows a window list button on the pane (for AuiNotebook). """
        aui.AUI_BUTTON_LEFT
        """ Shows a left button on the pane (for AuiNotebook). """
        aui.AUI_BUTTON_RIGHT
        """ Shows a right button on the pane (for AuiNotebook). """
        aui.AUI_BUTTON_UP
        """ Shows an up button on the pane (not implemented). """
        aui.AUI_BUTTON_DOWN
        """ Shows a down button on the pane (not implemented). """
        aui.AUI_BUTTON_CUSTOM1
        """ Shows a custom button on the pane. """
        aui.AUI_BUTTON_CUSTOM2
        """ Shows a custom button on the pane. """
        aui.AUI_BUTTON_CUSTOM3
        """ Shows a custom button on the pane. """
        aui.AUI_BUTTON_CUSTOM4
        """ Shows a custom button on the pane. """
        aui.AUI_BUTTON_CUSTOM5
        """ Shows a custom button on the pane. """
        aui.AUI_BUTTON_CUSTOM6
        """ Shows a custom button on the pane. """
        aui.AUI_BUTTON_CUSTOM7
        """ Shows a custom button on the pane. """
        aui.AUI_BUTTON_CUSTOM8
        """ Shows a custom button on the pane. """
        aui.AUI_BUTTON_CUSTOM9
        """ Shows a custom button on the pane. """

        # Pane Insert Level
        aui.AUI_INSERT_PANE
        """ Level for inserting a pane. """
        aui.AUI_INSERT_ROW
        """ Level for inserting a row. """
        aui.AUI_INSERT_DOCK
        """ Level for inserting a dock. """

        # ------------------------ #
        # - AuiToolBar Constants - #
        # ------------------------ #

        aui.ITEM_CONTROL
        """ The item in the AuiToolBar is a control. """
        aui.ITEM_LABEL
        """ The item in the AuiToolBar is a text label. """
        aui.ITEM_SPACER
        """ The item in the AuiToolBar is a spacer. """
        aui.ITEM_SEPARATOR
        """ The item in the AuiToolBar is a separator. """
        aui.ITEM_CHECK
        """ The item in the AuiToolBar is a toolbar check item. """
        aui.ITEM_NORMAL
        """ The item in the AuiToolBar is a standard toolbar item. """
        aui.ITEM_RADIO
        """ The item in the AuiToolBar is a toolbar radio item. """
        aui.ID_RESTORE_FRAME
        """ Identifier for restoring a minimized pane. """

        aui.BUTTON_DROPDOWN_WIDTH
        """ Width of the drop-down button in AuiToolBar. """

        aui.DISABLED_TEXT_GREY_HUE
        """ Hue text colour for the disabled text in AuiToolBar. """
        aui.DISABLED_TEXT_COLOUR
        """ Text colour for the disabled text in AuiToolBar. """

        aui.AUI_TB_TEXT
        """ Shows the text in the toolbar buttons; by default only icons are shown. """
        aui.AUI_TB_NO_TOOLTIPS
        """ Don't show tooltips on `AuiToolBar` items. """
        aui.AUI_TB_NO_AUTORESIZE
        """ Do not auto-resize the `AuiToolBar`. """
        aui.AUI_TB_GRIPPER
        """ Shows a gripper on the `AuiToolBar`. """
        aui.AUI_TB_OVERFLOW
        """ The `AuiToolBar` can contain overflow items. """
        aui.AUI_TB_VERTICAL
        """ The `AuiToolBar` is vertical. """
        aui.AUI_TB_HORZ_LAYOUT
        """ Shows the text and the icons alongside, not vertically stacked.
        This style must be used with ``AUI_TB_TEXT``. """
        aui.AUI_TB_PLAIN_BACKGROUND
        """ Don't draw a gradient background on the toolbar. """
        aui.AUI_TB_CLOCKWISE
        aui.AUI_TB_COUNTERCLOCKWISE

        aui.AUI_TB_HORZ_TEXT
        """ Combination of ``AUI_TB_HORZ_LAYOUT`` and ``AUI_TB_TEXT``. """
        aui.AUI_TB_VERT_TEXT

        aui.AUI_TB_DEFAULT_STYLE
        """ `AuiToolBar` default style. """

        # AuiToolBar settings
        aui.AUI_TBART_SEPARATOR_SIZE
        """ Separator size in AuiToolBar. """
        aui.AUI_TBART_GRIPPER_SIZE
        """ Gripper size in AuiToolBar. """
        aui.AUI_TBART_OVERFLOW_SIZE
        """ Overflow button size in AuiToolBar. """

        # AuiToolBar text orientation
        aui.AUI_TBTOOL_TEXT_LEFT
        """ Text in AuiToolBar items is aligned left. """
        aui.AUI_TBTOOL_TEXT_RIGHT
        """ Text in AuiToolBar items is aligned right. """
        aui.AUI_TBTOOL_TEXT_TOP
        """ Text in AuiToolBar items is aligned top. """
        aui.AUI_TBTOOL_TEXT_BOTTOM
        """ Text in AuiToolBar items is aligned bottom. """

        # AuiToolBar tool orientation
        aui.AUI_TBTOOL_HORIZONTAL
        aui.AUI_TBTOOL_VERT_CLOCKWISE
        aui.AUI_TBTOOL_VERT_COUNTERCLOCKWISE

        # ------------------------------- #
        # - AuiSwitcherDialog Constants - #
        # ------------------------------- #

        aui.SWITCHER_TEXT_MARGIN_X
        aui.SWITCHER_TEXT_MARGIN_Y


#---------------------------------------------------------------------------

if __name__ == '__main__':
    unittest.main()




            dlg = wx.MessageDialog(parent, message, caption)
            if dlg.SetYesNoLabels("&Quit", "&Don't quit"):
                dlg.SetMessage("What do you want to do?")
            else: # buttons have standard "Yes"/"No" values, so rephrase the question
                dlg.SetMessage("Do you really want to quit?")



#!/usr/bin/env python

import wx

#---------------------------------------------------------------------------

class TestPanel(wx.Panel):
    def __init__(self, parent, log):
        self.log = log
        wx.Panel.__init__(self, parent, -1)

        b = wx.Button(self, -1, "Create and Show a ColourDialog", (50,50))
        self.Bind(wx.EVT_BUTTON, self.OnButton, b)


    def OnButton(self, evt):
        dlg = wx.ColourDialog(self)

        # Ensure the full colour dialog is displayed,
        # not the abbreviated version.
        dlg.GetColourData().SetChooseFull(True)

        if dlg.ShowModal() == wx.ID_OK:

            # If the user selected OK, then the dialog's wx.ColourData will
            # contain valid information. Fetch the data ...
            data = dlg.GetColourData()

            # ... then do something with it. The actual colour data will be
            # returned as a three-tuple (r, g, b) in this particular case.
            color = data.GetColour().Get()
            self.log.WriteText('You selected: %s\n' % str(color))
            self.SetBackgroundColour(color)
            self.Refresh()

        # Once the dialog is destroyed, Mr. wx.ColourData is no longer your
        # friend. Don't use it again!
        dlg.Destroy()

#---------------------------------------------------------------------------


def runTest(frame, nb, log):
    win = TestPanel(nb, log)
    return win


#---------------------------------------------------------------------------


overview = """\
This class represents the colour chooser dialog.

Use of this dialog is a multi-stage process.

The actual information about how to display the dialog and the colors in the
dialog's 'registers' are contained in a wx.ColourData instance that is created by
the dialog at init time. Before displaying the dialog, you may alter these settings
to suit your needs. In the example, we set the dialog up to show the extended colour
data selection pane. Otherwise, only the more compact and less extensive colour
dialog is shown.  You may also preset the colour as well as other items.

If the user selects something and selects OK, then the wx.ColourData instance contains
the colour data that the user selected. Before destroying the dialog, retrieve the data.
<b>Do not try to retain the wx.ColourData instance.</b> It will probably not be valid
after the dialog is destroyed.

Along with he wx.ColourDialog documentation, see also the wx.ColourData documentation
for details.
"""

if __name__ == '__main__':
    import sys,os
    import run
    run.main(['', os.path.basename(sys.argv[0])] + sys.argv[1:])




import unittest
from unittests import wtc
import wx

import wx.lib.agw.ultimatelistctrl as ULC

#---------------------------------------------------------------------------

class lib_agw_ultimatelistctrl_Tests(wtc.WidgetTestCase):

    def test_lib_agw_ultimatelistctrlCtorReport(self):
        ulc = ULC.UltimateListCtrl(self.frame, agwStyle=wx.LC_REPORT)

    def test_lib_agw_ultimatelistctrlCtorIcon(self):
        ulc = ULC.UltimateListCtrl(self.frame, agwStyle=wx.LC_ICON)

    def test_lib_agw_ultimatelistctrlCtorList(self):
        ulc = ULC.UltimateListCtrl(self.frame, agwStyle=wx.LC_LIST)

    def test_lib_agw_ultimatelistctrlCtorVirtual(self):
        ulc = ULC.UltimateListCtrl(self.frame, agwStyle=wx.LC_REPORT|wx.LC_VIRTUAL)

    def test_lib_agw_thumbnailctrlStyles(self):
        ULC.ULC_VRULES
        ULC.ULC_HRULES
        ULC.ULC_ICON
        ULC.ULC_SMALL_ICON
        ULC.ULC_LIST
        ULC.ULC_REPORT
        ULC.ULC_ALIGN_TOP
        ULC.ULC_ALIGN_LEFT
        ULC.ULC_AUTOARRANGE
        ULC.ULC_VIRTUAL
        ULC.ULC_EDIT_LABELS
        ULC.ULC_NO_HEADER
        ULC.ULC_NO_SORT_HEADER
        ULC.ULC_SINGLE_SEL
        ULC.ULC_SORT_ASCENDING
        ULC.ULC_SORT_DESCENDING
        ULC.ULC_TILE
        ULC.ULC_NO_HIGHLIGHT
        ULC.ULC_STICKY_HIGHLIGHT
        ULC.ULC_STICKY_NOSELEVENT
        ULC.ULC_SEND_LEFTCLICK
        ULC.ULC_HAS_VARIABLE_ROW_HEIGHT
        ULC.ULC_AUTO_CHECK_CHILD
        ULC.ULC_AUTO_TOGGLE_CHILD
        ULC.ULC_AUTO_CHECK_PARENT
        ULC.ULC_SHOW_TOOLTIPS
        ULC.ULC_HOT_TRACKING
        ULC.ULC_BORDER_SELECT
        ULC.ULC_TRACK_SELECT
        ULC.ULC_HEADER_IN_ALL_VIEWS
        ULC.ULC_NO_FULL_ROW_SELECT
        ULC.ULC_FOOTER
        ULC.ULC_USER_ROW_HEIGHT

    def test_lib_agw_ultimatelistctrlEvents(self):
        wx.EVT_LIST_BEGIN_DRAG
        wx.EVT_LIST_BEGIN_RDRAG
        wx.EVT_LIST_BEGIN_LABEL_EDIT
        wx.EVT_LIST_END_LABEL_EDIT
        wx.EVT_LIST_DELETE_ITEM
        wx.EVT_LIST_DELETE_ALL_ITEMS
        wx.EVT_LIST_KEY_DOWN
        wx.EVT_LIST_INSERT_ITEM
        wx.EVT_LIST_COL_CLICK
        wx.EVT_LIST_COL_RIGHT_CLICK
        wx.EVT_LIST_COL_BEGIN_DRAG
        wx.EVT_LIST_COL_END_DRAG
        wx.EVT_LIST_COL_DRAGGING
        wx.EVT_LIST_ITEM_SELECTED
        wx.EVT_LIST_ITEM_DESELECTED
        wx.EVT_LIST_ITEM_RIGHT_CLICK
        wx.EVT_LIST_ITEM_MIDDLE_CLICK
        wx.EVT_LIST_ITEM_ACTIVATED
        wx.EVT_LIST_ITEM_FOCUSED
        wx.EVT_LIST_CACHE_HINT
        # in doc but does not exist???
        #wx.EVT_LIST_ITEM_CHECKING
        #wx.EVT_LIST_ITEM_CHECKED
        #wx.EVT_LIST_COL_CHECKING
        #wx.EVT_LIST_COL_CHECKED
        #wx.EVT_LIST_FOOTER_CHECKING
        #wx.EVT_LIST_FOOTER_CHECKED
        #wx.EVT_LIST_ITEM_HYPERLINK
        #wx.EVT_LIST_FOOTER_CLICK
        #wx.EVT_LIST_FOOTER_RIGHT_CLICK
        #wx.EVT_LIST_ITEM_LEFT_CLICK
        #wx.EVT_LIST_END_DRAG

#---------------------------------------------------------------------------

if __name__ == '__main__':
    unittest.main()




                item = tree.GetFirstItem()

                while item:
                    ok = item.IsOk()
                    item = tree.GetNextItem(item)

                    # Do something with every tree item ...




import unittest
from unittests import wtc
import wx
#import os; print(os.getpid())

#---------------------------------------------------------------------------

class display_Tests(wtc.WidgetTestCase):

    def test_display(self):
        d = wx.Display()
        r = d.GetClientArea()
        c = wx.Display.GetCount()
        self.assertTrue(c >= 1)
        m = d.GetModes()
        self.assertTrue(isinstance(m, wx.ArrayVideoModes))
        self.assertTrue(len(m) > 0)
        vm = m[0]
        self.assertTrue(isinstance(vm, wx.VideoMode))
        self.assertTrue(vm.IsOk())

#---------------------------------------------------------------------------


if __name__ == '__main__':
    unittest.main()



#---------------------------------------------------------------------------
# Name:        etg/layout.py
# Author:      Robin Dunn
#
# Created:     30-Nov-2010
# Copyright:   (c) 2010-2020 by Total Control Software
# License:     wxWindows License
#---------------------------------------------------------------------------

import etgtools
import etgtools.tweaker_tools as tools

PACKAGE   = "wx"
MODULE    = "_core"
NAME      = "layout"   # Base name of the file to generate to for this script
DOCSTRING = ""

# The classes and/or the basename of the Doxygen XML files to be processed by
# this script.
ITEMS  = [ 'wxIndividualLayoutConstraint',
           'wxLayoutConstraints'
           ]

#---------------------------------------------------------------------------

def run():
    # Parse the XML file(s) building a collection of Extractor objects
    module = etgtools.ModuleDef(PACKAGE, MODULE, NAME, DOCSTRING)
    etgtools.parseDoxyXML(module, ITEMS)

    #-----------------------------------------------------------------
    # Tweak the parsed meta objects in the module object as needed for
    # customizing the generated code and docstrings.


    c = module.find('wxIndividualLayoutConstraint')
    assert isinstance(c, etgtools.ClassDef)

    c.find('GetOtherWindow').setCppCode('return (wxWindow*)self->GetOtherWindow();')

    c.addProperty('Done GetDone SetDone')
    c.addProperty('Margin GetMargin SetMargin')
    c.addProperty('MyEdge GetMyEdge')
    c.addProperty('OtherEdge GetOtherEdge')
    c.addProperty('OtherWindow GetOtherWindow')
    c.addProperty('Percent GetPercent')
    c.addProperty('Relationship GetRelationship SetRelationship')
    c.addProperty('Value GetValue SetValue')


    #-----------------------------------------------------------------
    tools.doCommonTweaks(module)
    tools.runGenerators(module)


#---------------------------------------------------------------------------
if __name__ == '__main__':
    run()




import unittest
from unittests import wtc
import wx
import wx.adv
import os

imgFiles = [os.path.join(os.path.dirname(__file__), 'LB01.png'),
            os.path.join(os.path.dirname(__file__), 'LB02.png'),
            os.path.join(os.path.dirname(__file__), 'LB03.png'),
            os.path.join(os.path.dirname(__file__), 'LB04.png'),
            ]

#---------------------------------------------------------------------------

class bmpcbox_Tests(wtc.WidgetTestCase):

    @unittest.skipIf('wxMac' in wx.PlatformInfo, 'Needs a real MainLoop on wxMac')
    def test_bmpcbox1(self):
        pnl = wx.Panel(self.frame)

        bcb = wx.adv.BitmapComboBox(pnl)
        for idx, name in enumerate(imgFiles):
            bmp = wx.Bitmap(name)
            bcb.Append(os.path.basename(name), bmp)

        bcb.GetItemBitmap(0)
        bcb.SetItemBitmap(0, wx.NullBitmap)

        bcb.SetClientObject(2, "Hello")
        self.assertEqual(bcb.GetClientObject(2), "Hello")

        bcb.SetClientData(2, "Bye")
        self.assertEqual(bcb.GetClientData(2), "Bye")

        self.waitFor(300)


    @unittest.skipIf('wxMac' in wx.PlatformInfo, 'Needs a real MainLoop on wxMac')
    def test_bmpcbox2(self):
        pnl = wx.Panel(self.frame)

        bcb = wx.adv.BitmapComboBox(pnl)
        for idx, name in enumerate(imgFiles):
            bmp = wx.Bitmap(name)
            bcb.Append(os.path.basename(name), bmp, str(idx))

        self.assertEqual(bcb.GetClientData(2), "2")

        self.waitFor(300)


#---------------------------------------------------------------------------

if __name__ == '__main__':
    unittest.main()



import unittest
from unittests import wtc
import wx
import os

#---------------------------------------------------------------------------

class gauge_Tests(wtc.WidgetTestCase):

    def test_gauge(self):
        g1 = wx.Gauge(self.frame)
        g2 = wx.Gauge(self.frame, range=1000, style=wx.GA_VERTICAL)

    def test_gaugeProperties(self):
        g = wx.Gauge(self.frame)

        g.Range
        g.Value



#---------------------------------------------------------------------------


if __name__ == '__main__':
    unittest.main()



import unittest
from unittests import wtc
import wx
import wx.richtext

#---------------------------------------------------------------------------

class richtextstyles_Tests(wtc.WidgetTestCase):

    def test_richtextstyles01(self):
        ctrl = wx.richtext.RichTextStyleListCtrl(self.frame)


    def test_richtextstyles02(self):
        ctrl = wx.richtext.RichTextStyleListCtrl()
        ctrl.Create(self.frame)


    def test_richtextstyles03(self):
        with self.assertRaises(TypeError):
            sdef = wx.richtext.RichTextStyleDefinition()


    def test_richtextstyles04(self):
        sdef = wx.richtext.RichTextParagraphStyleDefinition()


    def test_richtextstyles05(self):
        ctrl = wx.richtext.RichTextStyleListBox(self.frame)


    def test_richtextstyles06(self):
        ctrl = wx.richtext.RichTextStyleListBox()
        ctrl.Create(self.frame)


    def test_richtextstyles05(self):
        wx.richtext.RichTextStyleListBox.RICHTEXT_STYLE_ALL
        wx.richtext.RichTextStyleListBox.RICHTEXT_STYLE_PARAGRAPH
        wx.richtext.RichTextStyleListBox.RICHTEXT_STYLE_CHARACTER
        wx.richtext.RichTextStyleListBox.RICHTEXT_STYLE_LIST
        wx.richtext.RichTextStyleListBox.RICHTEXT_STYLE_BOX


    def test_richtextstyles6(self):
        ctrl = wx.richtext.RichTextStyleComboCtrl(self.frame)


    def test_richtextstyles07(self):
        ctrl = wx.richtext.RichTextStyleComboCtrl()
        ctrl.Create(self.frame)


    def test_richtextstyles08(self):
        sdef = wx.richtext.RichTextCharacterStyleDefinition()


    def test_richtextstyles09(self):
        sdef = wx.richtext.RichTextListStyleDefinition()


    def test_richtextstyles10(self):
        sdef = wx.richtext.RichTextStyleSheet()


#---------------------------------------------------------------------------

if __name__ == '__main__':
    unittest.main()



import unittest
from unittests import wtc
import wx
import wx.aui

#---------------------------------------------------------------------------

class auibar_Tests(wtc.WidgetTestCase):

    def test_auibar01(self):
        wx.aui.AUI_TB_TEXT
        wx.aui.AUI_TB_NO_TOOLTIPS
        wx.aui.AUI_TB_NO_AUTORESIZE
        wx.aui.AUI_TB_GRIPPER
        wx.aui.AUI_TB_OVERFLOW
        wx.aui.AUI_TB_VERTICAL
        wx.aui.AUI_TB_HORZ_LAYOUT
        wx.aui.AUI_TB_HORIZONTAL
        wx.aui.AUI_TB_PLAIN_BACKGROUND
        wx.aui.AUI_TB_HORZ_TEXT
        wx.aui.AUI_ORIENTATION_MASK
        wx.aui.AUI_TB_DEFAULT_STYLE
        wx.aui.AUI_TBART_SEPARATOR_SIZE
        wx.aui.AUI_TBART_GRIPPER_SIZE
        wx.aui.AUI_TBART_OVERFLOW_SIZE
        wx.aui.AUI_TBTOOL_TEXT_LEFT
        wx.aui.AUI_TBTOOL_TEXT_RIGHT
        wx.aui.AUI_TBTOOL_TEXT_TOP
        wx.aui.AUI_TBTOOL_TEXT_BOTTOM



    def test_auibar02(self):
        evt = wx.aui.AuiToolBarEvent()


    def test_auibar03(self):
        wx.aui.wxEVT_AUITOOLBAR_TOOL_DROPDOWN
        wx.aui.wxEVT_AUITOOLBAR_OVERFLOW_CLICK
        wx.aui.wxEVT_AUITOOLBAR_RIGHT_CLICK
        wx.aui.wxEVT_AUITOOLBAR_MIDDLE_CLICK
        wx.aui.wxEVT_AUITOOLBAR_BEGIN_DRAG

        wx.aui.EVT_AUITOOLBAR_TOOL_DROPDOWN
        wx.aui.EVT_AUITOOLBAR_OVERFLOW_CLICK
        wx.aui.EVT_AUITOOLBAR_RIGHT_CLICK
        wx.aui.EVT_AUITOOLBAR_MIDDLE_CLICK
        wx.aui.EVT_AUITOOLBAR_BEGIN_DRAG



    def test_auibar04(self):
        obj = wx.aui.AuiToolBarItem()


    def test_auibar05(self):
        with self.assertRaises(TypeError):
            obj = wx.aui.AuiToolBarArt()


    def test_auibar06(self):
        obj = wx.aui.AuiDefaultToolBarArt()



    def test_auibar07(self):
        tbar = wx.aui.AuiToolBar(self.frame)


    def test_auibar08(self):
        tbar = wx.aui.AuiToolBar()
        tbar.Create(self.frame)



#---------------------------------------------------------------------------

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

import wx

#----------------------------------------------------------------------

class TestPanel(wx.Panel):
    def __init__(self, parent, log):
        self.log = log
        wx.Panel.__init__(self, parent, -1)

        # Create some controls
        self.ai = wx.ActivityIndicator(self)
        self.ai.Start()
        startBtn = wx.Button(self, label='Start')
        stopBtn = wx.Button(self, label='Stop')

        # Set up the layout
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(wx.StaticText(self, label='wx.ActivityIndicator: '),
                  wx.SizerFlags().CenterVertical())
        sizer.Add(self.ai, wx.SizerFlags().Border(wx.LEFT, 10))
        sizer.Add(startBtn, wx.SizerFlags().Border(wx.LEFT, 40))
        sizer.Add(stopBtn, wx.SizerFlags().Border(wx.LEFT, 10))

        # Put it all in an outer box with a border
        box = wx.BoxSizer()
        box.Add(sizer, wx.SizerFlags(1).Border(wx.ALL, 30))
        self.SetSizer(box)

        # Set up the event handlers
        self.Bind(wx.EVT_BUTTON, self.OnStart, startBtn)
        self.Bind(wx.EVT_BUTTON, self.OnStop, stopBtn)
        self.Bind(wx.EVT_UPDATE_UI, self.OnCheckBtnStatus, startBtn)
        self.Bind(wx.EVT_UPDATE_UI, self.OnCheckBtnStatus, stopBtn)


    def OnStart(self, evt):
        self.ai.Start()


    def OnStop(self, evt):
        self.ai.Stop()


    def OnCheckBtnStatus(self, evt):
        obj = evt.GetEventObject()
        running = self.ai.IsRunning()
        if obj.Label == 'Start':
            evt.Enable(not running)
        if obj.Label == 'Stop':
            evt.Enable(running)

#----------------------------------------------------------------------

def runTest(frame, nb, log):
    win = TestPanel(nb, log)
    return win

#----------------------------------------------------------------------



overview = """<html><body>
<h2><center>wx.ActivityIndicator</center></h2>

The wx.ActivityIndicator is a small platform-specifc control showing an
animation that can be used to indicate that the program is currently busy
performing some background task.

</body></html>
"""



if __name__ == '__main__':
    import sys,os
    import run
    run.main(['', os.path.basename(sys.argv[0])] + sys.argv[1:])




#!/usr/bin/env python

import wx


class TestPanel(wx.Panel):
    def __init__(self, parent, log):
        self.log = log
        wx.Panel.__init__(self, parent, -1)

        self.bannish = False
        self.annoy = True

        b = wx.Button(self, -1, "Create and Show a wx.RichMessageDialog", (50, 50))
        self.Bind(wx.EVT_BUTTON, self.OnButton, b)

    def OnButton(self, event):
        if self.bannish:
            print("That's rich!")
            return

        def DING():
            """Add some audacity...erm annoyance"""
            wx.Bell()

        event.Skip()
        dlg = wx.RichMessageDialog(self, "Hey You! I'm a rich irritating dialog!")
        dlg.ShowDetailedText(("bla bla bla " * 3) + '\n...\n\n' +
                              "Detailed Text(translated) == Don't show the welcome dialog again"
                              "\n(That almost always seems to popup...)")
        dlg.ShowCheckBox("Bannish me forever")
        dlg.ShowModal()  # return value ignored as we have "Ok" only anyhow

        for i in range(0, 10):
            wx.CallLater(500, DING)

        if dlg.IsCheckBoxChecked():
            # ... make sure we won't show it again the next time ...
            self.bannish = True  # Plonk!
            DING()
        dlg.Destroy()


#---------------------------------------------------------------------------


def runTest(frame, nb, log):
    win = TestPanel(nb, log)
    return win


#---------------------------------------------------------------------------


overview = """\
A RichMessageDialog...
Extension of MessageDialog with additional functionality.

This class adds the possibility of using a checkbox (that is especially useful
for implementing the "Don't ask me again" kind of dialogs) and an extra explanatory
text which is initially collapsed and not shown to the user but can be expanded to
show more information.

Notice that currently the native dialog is used only under MSW when using Vista or
later Windows version. Elsewhere, or for older versions of Windows, a generic
implementation which is less familiar to the users is used. Because of this it's
recommended to use this class only if you do need its extra functionality and use
MessageDialog which does have native implementation under all platforms otherwise.
However if you do need to put e.g. a checkbox in a dialog, you should definitely
consider using this class instead of using your own custom dialog because it will
have much better appearance at least under recent Windows versions.

To use this class, you need to create the dialog object and call ShowCheckBox and/or
ShowDetailedText to configure its contents. Other than that, it is used in exactly
the same way as MessageDialog and supports all the styles supported by it. In particular,
ShowModal return value is the same as for MessageDialog. The only difference is that you
need to use IsCheckBoxChecked to examine the checkbox value if you had called ShowCheckBox.
"""


if __name__ == '__main__':
    import sys
    import os
    import run
    run.main(['', os.path.basename(sys.argv[0])] + sys.argv[1:])



# -*- coding: utf-8 -*-
# ---------------------------------------------------------------------------
# Name:        etgtools/map_generator.py
# Author:      Robin Dunn
#
# Created:     20-May-2016
# Copyright:   (c) 2016-2020 by Total Control Software
# License:     wxWindows License
# ---------------------------------------------------------------------------

"""
This module provides a class that manages a persistent mapping, currently just
for mapping module item names to the name of their module, so other phases of
the code generation can find things that may not have been seen yet.  This
class can easily be adapted to other purposes however, if the need arises.
"""

# Standard library imports
import os.path as op
import json


# Phoenix imports
from .generators import textfile_open
from sphinxtools.constants import SPHINXROOT

# ---------------------------------------------------------------------------

class ItemModuleMap(object):
    """
    A persistent (across builds) mapping.  It manages the data in a
    dictionary, and also has a few methods to make the object quack a little
    like a real dictionary.
    """

    # This is the Borg pattern, so all instances of this class actually share
    # the same data attributes
    __shared_state = dict(_haveReadData=False,
                          _items=dict())

    def __init__(self):
        self.__dict__ = self.__shared_state # Borg part 2
        self.fileName = op.join(SPHINXROOT, 'itemToModuleMap.json')


    @property
    def items(self):
        # lazy load the items on first use
        if not self._haveReadData:
            self.read()
        return self._items


    # Methods for reading/writing the data from/to persistent storage.
    def read(self):
        if op.isfile(self.fileName):
            with textfile_open(self.fileName, 'rt') as fid:
                items = json.load(fid)
                # TODO: catch JSON exception...
                if items is None:
                    items = dict()
        else:
            items = dict()

        self._items.clear()
        self._items.update(items)
        self._haveReadData = True


    def flush(self):
        if not self._haveReadData and not self._items:
            return
        with textfile_open(self.fileName, 'wt') as fid:
            # Dump the data to a file in json, using a format that minimizes
            # excess whitespace.
            json.dump(self._items, fid, sort_keys=True,
                      indent=0, separators=(',', ':'))


    def reset(self):
        self._haveReadData = False,
        self._items.clear()


    def get_module(self, name):
        return self.items.get(name)

    def get_fullname(self, name):
        module = self.items.get(name)
        if not module:
            return name
        return module + name



    # Methods for a dictionary Facade, for convenience
    def get(self, key, default=None):
        return self.items.get(key, default)

    def clear(self):
        self.items.clear()

    def __len__(self):
        return len(self.items)

    def __getitem__(self, key):
        if key in self.items:
            return self.items[key]
        raise KeyError(key)

    def __setitem__(self, key, item):
        self.items[key] = item

    def __delitem__(self, key):
        del self.items[key]

    def __iter__(self):
        return iter(self.items)

    def __contains__(self, key):
        return key in self.items




# ---------------------------------------------------------------------------



#---------------------------------------------------------------------------
# Name:        etg/propgridpagestate.py
# Author:      Robin Dunn
#
# Created:     23-Feb-2015
# Copyright:   (c) 2015-2020 by Total Control Software
# License:     wxWindows License
#---------------------------------------------------------------------------

import etgtools
import etgtools.tweaker_tools as tools

PACKAGE   = "wx"
MODULE    = "_propgrid"
NAME      = "propgridpagestate"   # Base name of the file to generate to for this script
DOCSTRING = ""

# The classes and/or the basename of the Doxygen XML files to be processed by
# this script.
ITEMS  = [ 'wxPropertyGridHitTestResult',
           'wxPropertyGridIteratorBase',
           'wxPropertyGridIterator',
           'wxPGVIterator',
           'wxPropertyGridPageState',
           ]

#---------------------------------------------------------------------------

def run():
    # Parse the XML file(s) building a collection of Extractor objects
    module = etgtools.ModuleDef(PACKAGE, MODULE, NAME, DOCSTRING)
    etgtools.parseDoxyXML(module, ITEMS)

    #-----------------------------------------------------------------
    # Tweak the parsed meta objects in the module object as needed for
    # customizing the generated code and docstrings.

    c = module.find('wxPropertyGridHitTestResult')
    assert isinstance(c, etgtools.ClassDef)

    c = module.find('wxPropertyGridIterator')
    # TODO: Add python iterator methods to this and other iterator classes

    c = module.find('wxPGVIterator')
    c.find('wxPGVIterator').findOverload('wxPGVIteratorBase').ignore()

    c = module.find('wxPropertyGridPageState')
    tools.ignoreConstOverloads(c)
    # Incorrectly documented in 3.1.7
    c.find('GetColumnFullWidth.p').type = 'wxPGProperty *'


    module.find('wxPG_IT_CHILDREN').ignore()


    # Switch all wxVariant types to wxPGVariant, so the propgrid-specific
    # version of the MappedType will be used for converting to/from Python
    # objects.
    for item in module.allItems():
        if hasattr(item, 'type') and 'wxVariant' in item.type:
            item.type = item.type.replace('wxVariant', 'wxPGVariant')

    #-----------------------------------------------------------------
    tools.doCommonTweaks(module)
    tools.runGenerators(module)


#---------------------------------------------------------------------------
if __name__ == '__main__':
    run()




import unittest
from unittests import wtc
import wx
import six
from six import BytesIO as FileLikeObject
import os


pngFile = os.path.join(os.path.dirname(__file__), 'toucan.png')

#---------------------------------------------------------------------------

class stream_Tests(wtc.WidgetTestCase):

    def test_inputStreamParam(self):
        # This tests being able to pass a Python file-like object to a
        # wrapped function expecting a wxInputStream.

        # First, load the image data into a StringIO object
        with open(pngFile, 'rb') as f:
            stream = FileLikeObject(f.read())


        # Then use it to create a wx.Image
        img = wx.Image(stream)
        self.assertTrue(img.IsOk())

    def test_outputStreamParam(self):
        # This tests being able to pass a Python file-like object to a
        # wrapped function expecting a wxOutputStream.

        image = wx.Image(pngFile)
        stream = FileLikeObject()
        image.SaveFile(stream, wx.BITMAP_TYPE_PNG)
        del image

        stream = FileLikeObject(stream.getvalue())
        image = wx.Image(stream)
        self.assertTrue(image.IsOk())



#---------------------------------------------------------------------------


if __name__ == '__main__':
    unittest.main()
